@@ -4,7 +4,7 @@
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Tue Sep 28 07:01:42 1999
-# Version       : $Id: BinariesCheck.py 1559 2009-02-28 10:49:57Z scop $
+# Version       : $Id: BinariesCheck.py 1778 2010-04-27 21:35:57Z scop $
 # Purpose       : check binary files in a binary rpm package.
 #############################################################################
 
@@ -33,29 +33,27 @@ class BinaryInfo:
     #   GNU_STACK      0x000000 0x00000000 0x00000000 0x00000 0x00000 RWE 0x4
     stack_regex = re.compile('^\s+GNU_STACK\s+(?:(?:\S+\s+){5}(\S+)\s+)?')
     stack_exec_regex = re.compile('^..E$')
-    non_pic_regex = re.compile('TEXTREL', re.MULTILINE)
     undef_regex = re.compile('^undefined symbol:\s+(\S+)')
     unused_regex = re.compile('^\s+(\S+)')
-    debug_file_regex = re.compile('\.debug$')
     exit_call_regex = re.compile('\s+FUNC\s+.*?\s+(_?exit(?:@\S+)?)(?:\s|$)')
     fork_call_regex = re.compile('\s+FUNC\s+.*?\s+(fork(?:@\S+)?)(?:\s|$)')
 
     def __init__(self, pkg, path, file, is_ar, is_shlib):
-        self.readelf_error = 0
+        self.readelf_error = False
         self.needed = []
         self.rpath = []
         self.undef = []
         self.unused = []
-        self.comment = 0
-        self.soname = 0
-        self.non_pic = 1
-        self.stack = 0
-        self.exec_stack = 0
+        self.comment = False
+        self.soname = False
+        self.non_pic = True
+        self.stack = False
+        self.exec_stack = False
         self.exit_calls = []
-        fork_called = 0
+        fork_called = False
         self.tail = ''
 
-        is_debug = BinaryInfo.debug_file_regex.search(path)
+        is_debug = path.endswith('.debug')
 
         cmd = ['env', 'LC_ALL=C', 'readelf', '-W', '-S', '-l', '-d', '-s']
         cmd.append(path)
@@ -75,11 +73,11 @@ class BinaryInfo:
                     continue
 
                 if BinaryInfo.comment_regex.search(l):
-                    self.comment = 1
+                    self.comment = True
                     continue
 
                 if BinaryInfo.pic_regex.search(l):
-                    self.non_pic = 0
+                    self.non_pic = False
                     continue
 
                 r = BinaryInfo.soname_regex.search(l)
@@ -89,10 +87,10 @@ class BinaryInfo:
 
                 r = BinaryInfo.stack_regex.search(l)
                 if r:
-                    self.stack = 1
+                    self.stack = True
                     flags = r.group(1)
                     if flags and BinaryInfo.stack_exec_regex.search(flags):
-                        self.exec_stack = 1
+                        self.exec_stack = True
                     continue
 
                 if is_shlib:
@@ -102,11 +100,11 @@ class BinaryInfo:
                         continue
                     r = BinaryInfo.fork_call_regex.search(l)
                     if r:
-                        fork_called = 1
+                        fork_called = True
                         continue
 
             if self.non_pic:
-                self.non_pic = BinaryInfo.non_pic_regex.search(res[1])
+                self.non_pic = 'TEXTREL' in res[1]
 
             # Ignore all exit() calls if fork() is being called.
             # Does not have any context at all but without this kludge, the
@@ -115,7 +113,7 @@ class BinaryInfo:
                 self.exit_calls = []
 
         else:
-            self.readelf_error = 1
+            self.readelf_error = True
             printWarning(pkg, 'binaryinfo-readelf-failed',
                          file, re.sub('\n.*', '', res[1]))
 
@@ -148,28 +146,22 @@ class BinaryInfo:
             if res[0]:
                 # Either ldd doesn't grok -u (added in glibc 2.3.4) or we have
                 # unused direct dependencies
-                in_unused = 0
+                in_unused = False
                 for l in res[1].splitlines():
                     if not l.rstrip():
                         pass
                     elif l.startswith('Unused direct dependencies'):
-                        in_unused = 1
+                        in_unused = True
                     elif in_unused:
                         unused = BinaryInfo.unused_regex.search(l)
                         if unused:
                             self.unused.append(unused.group(1))
                         else:
-                            in_unused = 0
+                            in_unused = False
 
 path_regex = re.compile('(.*/)([^/]+)')
 numeric_dir_regex = re.compile('/usr(?:/share)/man/man./(.*)\.[0-9](?:\.gz|\.bz2)')
 versioned_dir_regex = re.compile('[^.][0-9]')
-usr_share = re.compile('^/usr/share/')
-etc = re.compile('^/etc/')
-not_stripped = re.compile('not stripped')
-unstrippable = re.compile('\.o$|\.static$')
-shared_object_regex = re.compile('shared object')
-executable_regex = re.compile('executable')
 libc_regex = re.compile('libc\.')
 ldso_soname_regex = re.compile('^ld(-linux(-(ia|x86_)64))?\.so')
 so_regex = re.compile('/lib(64)?/[^/]+\.so(\.[0-9]+)*$')
@@ -219,154 +211,176 @@ class BinariesCheck(AbstractCheck.AbstractCheck):
 
             if not stat.S_ISDIR(pkgfile.mode) and usr_lib_regex.search(fname):
                 has_usr_lib_file = True
-                if usr_lib_exception_regex.search(fname):
+                if not binary_in_usr_lib and \
+                        usr_lib_exception_regex.search(fname):
                     # Fake that we have binaries there to avoid
                     # only-non-binary-in-usr-lib false positives
                     binary_in_usr_lib = True
 
-            is_elf = pkgfile.magic.find('ELF') != -1
-            is_ar = pkgfile.magic.find('current ar archive') != -1
-            is_ocaml_native = pkgfile.magic.find('Objective caml native') != -1
+            is_elf = 'ELF' in pkgfile.magic
+            is_ar = 'current ar archive' in pkgfile.magic
+            is_ocaml_native = 'Objective caml native' in pkgfile.magic
             is_binary = is_elf or is_ar or is_ocaml_native
-            is_shlib = so_regex.search(fname)
 
-            if is_binary:
-                binary = True
-                if has_usr_lib_file and not binary_in_usr_lib and usr_lib_regex.search(fname):
-                    binary_in_usr_lib = True
-
-                if pkg.arch == 'noarch':
-                    printError(pkg, 'arch-independent-package-contains-binary-or-object', fname)
-                else:
-                    # in /usr/share ?
-                    if usr_share.search(fname):
-                        printError(
-                            pkg, 'arch-dependent-file-in-usr-share', fname)
-                    # in /etc ?
-                    if etc.search(fname):
-                        printError(pkg, 'binary-in-etc', fname)
-
-                    if pkg.arch == 'sparc' and \
-                           sparc_regex.search(pkgfile.magic):
-                        printError(pkg, 'non-sparc32-binary', fname)
-
-                    # stripped ?
-                    if not unstrippable.search(fname) and not is_ocaml_native:
-                        if not_stripped.search(pkgfile.magic):
-                            printWarning(
-                                pkg, 'unstripped-binary-or-object', fname)
-
-                        # inspect binary file
-                        bin_info = BinaryInfo(
-                            pkg, pkgfile.path, fname, is_ar, is_shlib)
-
-                        # so name in library
-                        if is_shlib:
-                            has_lib = True
-                            if bin_info.readelf_error:
-                                pass
-                            elif not bin_info.soname:
-                                printWarning(pkg, 'no-soname', fname)
-                            else:
-                                if not validso_regex.search(bin_info.soname):
-                                    printError(pkg, 'invalid-soname', fname,
-                                               bin_info.soname)
-                                else:
-                                    (directory, base) = dir_base(fname)
-                                    try:
-                                        symlink = directory + bin_info.soname
-                                        link = files[symlink].linkto
-                                        if link not in (fname, base, ''):
-                                            printError(pkg, 'invalid-ldconfig-symlink', fname, link)
-                                    except KeyError:
-                                        printError(
-                                            pkg, 'no-ldconfig-symlink', fname)
-                                res = soversion_regex.search(bin_info.soname)
-                                if res:
-                                    soversion = res.group(1) or res.group(2)
-                                    if version is None:
-                                        version = soversion
-                                    elif version != soversion:
-                                        version = -1
-
-                            if bin_info.non_pic and not bin_info.readelf_error:
-                                printError(
-                                    pkg, 'shlib-with-non-pic-code', fname)
-
-                        # rpath ?
-                        if bin_info.rpath:
-                            for p in bin_info.rpath:
-                                if p in system_lib_paths or \
-                                   not usr_lib_regex.search(p):
-                                    printError(pkg, 'binary-or-shlib-defines-rpath', fname, bin_info.rpath)
-                                    break
-
-                        # shared lib calls exit() or _exit()?
-                        if is_shlib and bin_info.exit_calls:
-                            for ec in bin_info.exit_calls:
-                                printWarning(
-                                    pkg, 'shared-lib-calls-exit', fname, ec)
-
-                        is_exec = executable_regex.search(pkgfile.magic)
-                        if is_exec or \
-                               shared_object_regex.search(pkgfile.magic):
-
-                            if is_exec:
-
-                                if bin_regex.search(fname):
-                                    exec_files.append(fname)
-
-                                if ocaml_mixed_regex.search(bin_info.tail):
-                                    printWarning(
-                                        pkg, 'ocaml-mixed-executable', fname)
-
-                            if bin_info.readelf_error:
-                                pass
-                            elif not bin_info.needed and \
-                               not (bin_info.soname and \
-                                    ldso_soname_regex.search(bin_info.soname)):
-                                if shared_object_regex.search(pkgfile.magic):
-                                    printError(pkg, 'shared-lib-without-dependency-information', fname)
-                                else:
-                                    printError(pkg, 'statically-linked-binary', fname)
-                            else:
-                                # linked against libc ?
-                                if not libc_regex.search(fname) and \
-                                   ( not bin_info.soname or \
-                                     ( not libc_regex.search(bin_info.soname) and \
-                                       not ldso_soname_regex.search(bin_info.soname))):
-                                    found_libc = 0
-                                    for lib in bin_info.needed:
-                                        if libc_regex.search(lib):
-                                            found_libc = 1
-                                            break
-                                    if not found_libc:
-                                        if shared_object_regex.search(pkgfile.magic):
-                                            printError(pkg, 'library-not-linked-against-libc', fname)
-                                        else:
-                                            printError(pkg, 'program-not-linked-against-libc', fname)
-
-                            # It could be useful to check these for others than
-                            # shared libs only, but that has potential to
-                            # generate lots of false positives and noise.
-                            if is_shlib:
-                                for s in bin_info.undef:
-                                    printWarning(pkg, 'undefined-non-weak-symbol', fname, s)
-                                for s in bin_info.unused:
-                                    printWarning(pkg, 'unused-direct-shlib-dependency', fname, s)
-
-                            if bin_info.stack:
-                                if bin_info.exec_stack:
-                                    printWarning(pkg, 'executable-stack', fname)
-                            elif pkg.arch.endswith("86") or pkg.arch.startswith("pentium") or pkg.arch in ("athlon", "x86_64"):
-                                printError(pkg, 'missing-PT_GNU_STACK-section', fname)
-
-            else:
+            if not is_binary:
                 if reference_regex.search(fname):
                     lines = pkg.grep(invalid_dir_ref_regex, fname)
                     if lines:
                         printError(pkg, 'invalid-directory-reference', fname,
                                    '(line %s)' % ", ".join(lines))
+                continue
+
+            # binary files only from here on
+
+            binary = True
+
+            if has_usr_lib_file and not binary_in_usr_lib and \
+                    usr_lib_regex.search(fname):
+                binary_in_usr_lib = True
+
+            if pkg.arch == 'noarch':
+                printError(
+                    pkg,
+                    'arch-independent-package-contains-binary-or-object',
+                    fname)
+                continue
+
+            # arch dependent packages only from here on
+
+            # in /usr/share ?
+            if fname.startswith('/usr/share/'):
+                printError(pkg, 'arch-dependent-file-in-usr-share', fname)
+
+            # in /etc ?
+            if fname.startswith('/etc/'):
+                printError(pkg, 'binary-in-etc', fname)
+
+            if pkg.arch == 'sparc' and sparc_regex.search(pkgfile.magic):
+                printError(pkg, 'non-sparc32-binary', fname)
+
+            if is_ocaml_native or fname.endswith('.o') or \
+                    fname.endswith('.static'):
+                continue
+
+            # stripped ?
+            if 'not stripped' in pkgfile.magic:
+                printWarning(pkg, 'unstripped-binary-or-object', fname)
+
+            # inspect binary file
+            is_shlib = so_regex.search(fname)
+            bin_info = BinaryInfo(pkg, pkgfile.path, fname, is_ar, is_shlib)
+
+            if is_shlib:
+                has_lib = True
+
+            # shared libs
+            if is_shlib and not bin_info.readelf_error:
+
+                # so name in library
+                if not bin_info.soname:
+                    printWarning(pkg, 'no-soname', fname)
+                else:
+                    if not validso_regex.search(bin_info.soname):
+                        printError(pkg, 'invalid-soname', fname,
+                                   bin_info.soname)
+                    else:
+                        (directory, base) = dir_base(fname)
+                        try:
+                            symlink = directory + bin_info.soname
+                            link = files[symlink].linkto
+                            if link not in (fname, base, ''):
+                                printError(pkg, 'invalid-ldconfig-symlink',
+                                           fname, link)
+                        except KeyError:
+                            if base.startswith("lib") or base.startswith("ld-"):
+                                printError(pkg, 'no-ldconfig-symlink', fname)
+
+                    res = soversion_regex.search(bin_info.soname)
+                    if res:
+                        soversion = res.group(1) or res.group(2)
+                        if version is None:
+                            version = soversion
+                        elif version != soversion:
+                            version = -1
+
+                if bin_info.non_pic:
+                    printError(pkg, 'shlib-with-non-pic-code', fname)
+
+                # It could be useful to check these for others than shared
+                # libs only, but that has potential to generate lots of
+                # false positives and noise.
+                for s in bin_info.undef:
+                    printWarning(pkg, 'undefined-non-weak-symbol', fname, s)
+                for s in bin_info.unused:
+                    printWarning(pkg, 'unused-direct-shlib-dependency',
+                                 fname, s)
+
+                # calls exit() or _exit()?
+                for ec in bin_info.exit_calls:
+                    printWarning(pkg, 'shared-lib-calls-exit', fname, ec)
+
+            # rpath ?
+            if bin_info.rpath:
+                for p in bin_info.rpath:
+                    if p in system_lib_paths or not usr_lib_regex.search(p):
+                        printError(pkg, 'binary-or-shlib-defines-rpath',
+                                   fname, bin_info.rpath)
+                        break
+
+            is_exec = 'executable' in pkgfile.magic
+            is_shobj = 'shared object' in pkgfile.magic
+
+            if not is_exec and not is_shobj:
+                continue
+
+            if is_exec:
+
+                if bin_regex.search(fname):
+                    exec_files.append(fname)
+
+                if ocaml_mixed_regex.search(bin_info.tail):
+                    printWarning(pkg, 'ocaml-mixed-executable', fname)
+
+            if bin_info.readelf_error:
+                continue
+
+            if not bin_info.needed and not (
+                bin_info.soname and ldso_soname_regex.search(bin_info.soname)):
+                if is_shobj:
+                    printError(pkg,
+                               'shared-lib-without-dependency-information',
+                               fname)
+                else:
+                    printError(pkg, 'statically-linked-binary', fname)
+
+            else:
+                # linked against libc ?
+                if not libc_regex.search(fname) and \
+                        (not bin_info.soname or \
+                             (not libc_regex.search(bin_info.soname) and \
+                              not ldso_soname_regex.search(bin_info.soname))):
+
+                    found_libc = False
+                    for lib in bin_info.needed:
+                        if libc_regex.search(lib):
+                            found_libc = True
+                            break
+
+                    if not found_libc:
+                        if is_shobj:
+                            printError(pkg, 'library-not-linked-against-libc',
+                                       fname)
+                        else:
+                            printError(pkg, 'program-not-linked-against-libc',
+                                       fname)
+
+            if bin_info.stack:
+                if bin_info.exec_stack:
+                    printWarning(pkg, 'executable-stack', fname)
+            elif not bin_info.readelf_error and (
+                pkg.arch.endswith("86") or pkg.arch.startswith("pentium") or
+                pkg.arch in ("athlon", "x86_64")):
+                printError(pkg, 'missing-PT_GNU_STACK-section', fname)
 
         if has_lib:
             for f in exec_files:
@@ -374,9 +388,10 @@ class BinariesCheck(AbstractCheck.AbstractCheck):
             for f in files:
                 res = numeric_dir_regex.search(f)
                 fn = res and res.group(1) or f
-                if not f in exec_files and not so_regex.search(f) and not versioned_dir_regex.search(fn):
+                if f not in exec_files and not so_regex.search(f) and \
+                        not versioned_dir_regex.search(fn):
                     printError(pkg, 'non-versioned-file-in-library-package', f)
-            if version and version != -1 and pkg.name.find(version) == -1:
+            if version and version != -1 and version not in pkg.name:
                 printError(pkg, 'incoherent-version-in-name', version)
 
         if not binary and not multi_pkg and pkg.arch != 'noarch':
@@ -477,7 +492,8 @@ indicate gratuitously bloated linkage; check that the binary has been linked
 with the intended shared libraries only.''',
 
 'only-non-binary-in-usr-lib',
-'''There are only non binary files in /usr/lib so they should be in /usr/share.''',
+'''There are only non binary files in /usr/lib so they should be in
+/usr/share.''',
 
 'binaryinfo-readelf-failed',
 '''Executing readelf on this file failed, all checks could not be run.''',
