@@ -4,7 +4,7 @@
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Fri Oct 15 20:04:25 1999
-# Version       : $Id: Config.py 1517 2009-01-26 23:10:01Z scop $
+# Version       : $Id: Config.py 1732 2010-02-21 11:28:42Z scop $
 # Purpose       : handle configuration options. To be used from config files.
 #############################################################################
 
@@ -12,6 +12,10 @@ import locale
 import os.path
 import re
 
+try:
+    from __version__ import __version__
+except ImportError:
+    __version__ = 'devel'
 
 DEFAULT_CHECKS = ("DistributionCheck",
                   "TagsCheck",
@@ -45,8 +49,8 @@ except:
     except:
         pass
 
-info = 0
-no_exception = 0
+info = False
+no_exception = False
 
 # handle the list of checks to load
 _checks = []
@@ -56,7 +60,7 @@ def addCheck(check):
     global _checks
 
     check = re.sub('\.py[co]?$', '', check)
-    if not check in _checks:
+    if check not in _checks:
         _checks.append(check)
 
 def allChecks():
@@ -84,7 +88,7 @@ def addCheckDir(dir):
 
     d = os.path.expanduser(dir)
 
-    if not d in _dirs:
+    if d not in _dirs:
         _dirs.insert(0, d)
 
 def checkDirs():
@@ -147,21 +151,22 @@ def isFiltered(s):
     if _filters_re == None:
         # no filter
         if len(_filters) == 0:
-            return 0
+            return False
         _filters_re = '(?:' + _filters[0] + ')'
 
         for idx in range(1, len(_filters)):
-            # to prevent named group overflow that happen when there is too many () in a single regexp
-            # AssertionError: sorry, but this version only supports 100 named groups
-            if _filters[idx].find('(') > -1:
+            # to prevent named group overflow that happen when there is too
+            # many () in a single regexp: AssertionError: sorry, but this
+            # version only supports 100 named groups
+            if '(' in _filters[idx]:
                 _non_named_group_re.subn('(:?', _filters[idx])
             _filters_re = _filters_re + '|(?:' + _filters[idx] +')'
         _filters_re = re.compile(_filters_re)
 
     if not no_exception:
         if _filters_re.search(s):
-            return 1
-    return 0
+            return True
+    return False
 
 # Config.py ends here
 
