@@ -20,30 +20,20 @@ from Filter import addDetails, printWarning
 import AbstractCheck
 
 
-# remove version from deps like ['foo', '>', '3']
-def _stripVersionedDeps(deps):
-    deps = deps[:]
-    j    = len(deps)
-    while j >= 3:
-        j = j-1
-        if deps[j-1] in ('<', '<=', '=', '>=', '>'):
-            del deps[j-1:j+1]
-
-    return deps
-
 class DocFilesCheck(AbstractCheck.AbstractCheck):
     def __init__(self):
         AbstractCheck.AbstractCheck.__init__(self, 'DocFilesCheck')
 
     def __checkRequirements(self, pkg):
-        files = pkg.files()
         doc_files = pkg.docFiles()
+        if not doc_files:
+            return
+
+        files = pkg.files()
 
         reqs = {}
         for fname, pkgfile in files.items():
-            tmp = pkgfile.deps.split()
-            tmp = _stripVersionedDeps(tmp)
-            reqs[fname] = tmp
+            reqs[fname] = [x[0] for x in pkgfile.requires]
 
         core_reqs = {}  # dependencies of non-doc files
         doc_reqs  = {}  # dependencies of doc files
@@ -66,10 +56,8 @@ class DocFilesCheck(AbstractCheck.AbstractCheck):
             for r in reqs[i]:
                 if r not in target:
                     target[r] = []
-
                 target[r].append(i)
 
-
         # go through the calculated requirements of the %doc files
         for (dep, req_files) in doc_reqs.items():
             if dep not in core_reqs:
