@@ -4,7 +4,7 @@
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Sat Oct 23 15:52:27 1999
-# Version       : $Id: Filter.py 1533 2009-01-30 22:36:11Z scop $
+# Version       : $Id: Filter.py 1734 2010-02-23 20:19:22Z scop $
 # Purpose       : filter the output of rpmlint to allow exceptions.
 #############################################################################
 
@@ -22,6 +22,14 @@ _diagnostic = list()
 _badness_score = 0
 printed_messages = { "I": 0, "W": 0, "E": 0 }
 
+if sys.stdout.isatty():
+    def __print(s):
+        print(s)
+else:
+    import locale
+    def __print(s):
+        print(s.encode(locale.getpreferredencoding(), "replace"))
+
 def printInfo(pkg, reason, *details):
     _print("I", pkg, reason, details)
 
@@ -61,51 +69,33 @@ def _print(msgtype, pkg, reason, details):
             if threshold >= 0:
                 _diagnostic.append(s + "\n")
             else:
-                sys.stdout.write(s)
-                sys.stdout.write("\n")
+                __print(s)
                 if Config.info:
                     printDescriptions(reason)
-            return 1
+            return True
 
-    return 0
+    return False
 
 def printDescriptions(reason):
     try:
         d = _details[reason]
         if d and d != '' and d != "\n":
-            sys.stdout.write(textwrap.fill(d, 78))
-            sys.stdout.write("\n\n")
+            __print(textwrap.fill(d, 78))
+            __print("")
     except KeyError:
         pass
 
-def _diag_compare(x, y):
-
-    where_a = x.split()[1]
-    level_a = x.split()[2]
-
-    where_b = y.split()[1]
-    level_b = y.split()[2]
-
-    if level_b > level_a:
-        return 1
-    elif level_b == level_a:
-        if where_b > where_b:
-            return 1
-        elif where_b == where_a:
-            return 0
-        else:
-            return -1
-    else:
-        return -1
-
+def _diag_sortkey(x):
+    xs = x.split()
+    return (xs[2], xs[1])
 
 def printAllReasons():
     threshold = badnessThreshold()
     if threshold < 0:
-        return 0
+        return False
 
     global _badness_score, _diagnostic
-    _diagnostic.sort(_diag_compare)
+    _diagnostic.sort(key = _diag_sortkey, reverse = True)
     last_reason = ''
     for diag in _diagnostic:
         if Config.info:
@@ -114,7 +104,7 @@ def printAllReasons():
                 if len(last_reason):
                     printDescriptions(last_reason)
                 last_reason = reason
-        sys.stdout.write(diag)
+        __print(diag)
     if Config.info and len(last_reason):
         printDescriptions(last_reason)
     _diagnostic = list()
