@@ -3,7 +3,7 @@
 # Project         : Mandriva Linux
 # Module          : rpmlint
 # File            : MenuCheck.py
-# Version         : $Id: MenuCheck.py 1544 2009-02-01 22:05:18Z scop $
+# Version         : $Id: MenuCheck.py 1754 2010-03-30 20:34:48Z scop $
 # Author          : Frederic Lepied
 # Created On      : Mon Mar 20 07:43:37 2000
 #---------------------------------------------------------------
@@ -148,8 +148,6 @@ xpm_ext_regex = re.compile('/usr/share/icons/(mini/|large/).*\.xpm$')
 icon_ext_regex = re.compile(Config.getOption('IconFilename', '.*\.png$'))
 version_regex = re.compile('([0-9.][0-9.]+)($|\s)')
 launchers = Config.getOption('MenuLaunchers', DEFAULT_LAUNCHERS)
-bad_title_regex = re.compile('/')
-menu64_file_regex = re.compile('^/usr/lib64/menu')
 xdg_migrated_regex = re.compile('xdg=\"?([^\" ]+)')
 
 # compile regexps
@@ -198,82 +196,91 @@ class MenuCheck(AbstractCheck.AbstractCheck):
                     if res:
                         if stat.S_ISREG(mode) and not pkg.grep('None",', fname):
                             printWarning(pkg, 'non-transparent-xpm', fname)
-                if menu64_file_regex.search(fname):
+                if fname.startswith('/usr/lib64/menu'):
                     printError(pkg, 'menu-in-wrong-dir', fname)
 
         if menus:
             postin = pkg[rpm.RPMTAG_POSTIN] or pkg[rpm.RPMTAG_POSTINPROG]
             if not postin:
                 printError(pkg, 'menu-without-postin')
-            else:
-                if not update_menus_regex.search(postin):
-                    printError(pkg, 'postin-without-update-menus')
+            elif not update_menus_regex.search(postin):
+                printError(pkg, 'postin-without-update-menus')
 
             postun = pkg[rpm.RPMTAG_POSTUN] or pkg[rpm.RPMTAG_POSTUNPROG]
             if not postun:
                 printError(pkg, 'menu-without-postun')
-            else:
-                if not update_menus_regex.search(postun):
-                    printError(pkg, 'postun-without-update-menus')
+            elif not update_menus_regex.search(postun):
+                printError(pkg, 'postun-without-update-menus')
 
             directory = pkg.dirName()
             for f in menus:
                 # remove comments and handle cpp continuation lines
-                cmd = Pkg.getstatusoutput(('/lib/cpp', directory + f), 1)[1]
+                cmd = Pkg.getstatusoutput(('/lib/cpp', directory + f), True)[1]
 
                 for line in cmd.splitlines():
-                    if not line.startswith('?'): continue
+                    if not line.startswith('?'):
+                        continue
                     res = package_regex.search(line)
                     if res:
                         package = res.group(1)
                         if package != pkg.name:
-                            printWarning(pkg, 'incoherent-package-value-in-menu', package, f)
+                            printWarning(pkg,
+                                         'incoherent-package-value-in-menu',
+                                         package, f)
                     else:
                         printInfo(pkg, 'unable-to-parse-menu-entry', line)
 
-                    command = 1
+                    command = True
                     res = command_regex.search(line)
                     if res:
                         command_line = (res.group(1) or res.group(2)).split()
                         command = command_line[0]
                         for launcher in launchers:
-                            if launcher[0].search(command):
-                                found = 0
-                                if launcher[1]:
-                                    if ('/bin/' + command_line[0] in files or
-                                        '/usr/bin/' + command_line[0] in files or
-                                        '/usr/X11R6/bin/' + command_line[0] in files):
-                                        found = 1
-                                    else:
-                                        for l in launcher[1]:
-                                            if l in pkg.req_names():
-                                                found = 1
-                                                break
-                                    if not found:
-                                        printError(pkg, 'use-of-launcher-in-menu-but-no-requires-on', launcher[1][0])
-                                command = command_line[1]
-                                break
+                            if not launcher[0].search(command):
+                                continue
+                            found = False
+                            if launcher[1]:
+                                if ('/bin/' + command_line[0] in files or
+                                    '/usr/bin/' + command_line[0] in files
+                                    or '/usr/X11R6/bin/' + command_line[0]
+                                    in files):
+                                    found = True
+                                else:
+                                    for l in launcher[1]:
+                                        if l in pkg.req_names():
+                                            found = True
+                                            break
+                                if not found:
+                                    printError(pkg,
+                                               'use-of-launcher-in-menu-but-no-requires-on',
+                                               launcher[1][0])
+                            command = command_line[1]
+                            break
+
                         if command[0] == '/':
-                            if not command in files:
-                                printWarning(pkg, 'menu-command-not-in-package', command)
-                        else:
-                            if not ('/bin/' + command in files or
-                                    '/usr/bin/' + command in files or
-                                    '/usr/X11R6/bin/' + command in files):
-                                printWarning(pkg, 'menu-command-not-in-package', command)
+                            if command not in files:
+                                printWarning(pkg, 'menu-command-not-in-package',
+                                             command)
+                        elif not ('/bin/' + command in files or
+                                  '/usr/bin/' + command in files or
+                                  '/usr/X11R6/bin/' + command in files):
+                            printWarning(pkg, 'menu-command-not-in-package',
+                                         command)
                     else:
                         printWarning(pkg, 'missing-menu-command')
-                        command = 0
+                        command = False
 
                     res = longtitle_regex.search(line)
                     if res:
                         grp = res.groups()
                         title = grp[1] or grp[2]
                         if title[0] != title[0].upper():
-                            printWarning(pkg, 'menu-longtitle-not-capitalized', title)
+                            printWarning(pkg, 'menu-longtitle-not-capitalized',
+                                         title)
                         res = version_regex.search(title)
                         if res:
-                            printWarning(pkg, 'version-in-menu-longtitle', title)
+                            printWarning(pkg, 'version-in-menu-longtitle',
+                                         title)
                     else:
                         printError(pkg, 'no-longtitle-in-menu', f)
                         title = None
@@ -283,11 +290,12 @@ class MenuCheck(AbstractCheck.AbstractCheck):
                         grp = res.groups()
                         title = grp[1] or grp[2]
                         if title[0] != title[0].upper():
-                            printWarning(pkg, 'menu-title-not-capitalized', title)
+                            printWarning(pkg, 'menu-title-not-capitalized',
+                                         title)
                         res = version_regex.search(title)
                         if res:
                             printWarning(pkg, 'version-in-menu-title', title)
-                        if bad_title_regex.search(title):
+                        if '/' in title:
                             printError(pkg, 'invalid-title', title)
                     else:
                         printError(pkg, 'no-title-in-menu', f)
@@ -303,11 +311,12 @@ class MenuCheck(AbstractCheck.AbstractCheck):
                                 grp = res.groups()
                                 section = grp[1] or grp[2]
                                 # don't warn entries for sections
-                                if command:
-                                    if section not in valid_sections:
-                                        printError(pkg, 'invalid-menu-section', section, f)
+                                if command and section not in valid_sections:
+                                    printError(pkg, 'invalid-menu-section',
+                                               section, f)
                             else:
-                                printInfo(pkg, 'unable-to-parse-menu-section', line)
+                                printInfo(pkg, 'unable-to-parse-menu-section',
+                                          line)
                         elif needs not in standard_needs:
                             printInfo(pkg, 'strange-needs', needs, f)
                     else:
@@ -319,11 +328,14 @@ class MenuCheck(AbstractCheck.AbstractCheck):
                         if not icon_ext_regex.search(icon):
                             printWarning(pkg, 'invalid-menu-icon-type', icon)
                         if icon[0] == '/' and needs == 'x11':
-                            printWarning(pkg, 'hardcoded-path-in-menu-icon', icon)
+                            printWarning(pkg, 'hardcoded-path-in-menu-icon',
+                                         icon)
                         else:
                             for path in icon_paths:
-                                if not (path[0] + icon) in files:
-                                    printError(pkg, path[1] + '-icon-not-in-package', icon, f)
+                                if (path[0] + icon) not in files:
+                                    printError(pkg,
+                                               path[1] + '-icon-not-in-package',
+                                               icon, f)
                     else:
                         printWarning(pkg, 'no-icon-in-menu', title)
 
