@@ -4,7 +4,7 @@
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Tue Sep 28 07:18:06 1999
-# Version       : $Id: Pkg.py 1567 2009-03-17 23:20:11Z scop $
+# Version       : $Id: Pkg.py 1794 2010-06-05 19:55:53Z scop $
 # Purpose       : provide an API to handle a rpm package either by accessing
 #                 the rpm file or by accessing the files contained inside.
 #############################################################################
@@ -20,18 +20,29 @@ import subprocess
 
 try:
     import magic
+    # TODO: magic.MAGIC_COMPRESS when PkgFile gets decompress support.
     _magic = magic.open(magic.MAGIC_NONE)
     _magic.load()
 except:
     _magic = None
 import rpm
 
-from Filter import printWarning
+import Filter
+
+# Python 2/3 compatibility/convenience wrapper for printing to stderr with
+# less concerns of UnicodeErrors than plain sys.stderr.write.
+if sys.version_info[0] > 2:
+    # Blows up with Python < 3 without the exec() hack
+    exec('def warn(s): print (s, file=sys.stderr)')
+else:
+    def warn(s): print >> sys.stderr, s
 
 
 # utilities
 
-PREREQ_FLAG = rpm.RPMSENSE_PREREQ | \
+# 64: RPMSENSE_PREREQ is 0 with recent rpm versions, we want 64 here in order
+# to do the right thing with packages built with older rpm versions
+PREREQ_FLAG = (rpm.RPMSENSE_PREREQ or 64) | \
               rpm.RPMSENSE_SCRIPT_PRE | \
               rpm.RPMSENSE_SCRIPT_POST | \
               rpm.RPMSENSE_SCRIPT_PREUN | \
@@ -58,17 +69,21 @@ def substitute_shell_vars(val, script):
         value = shell_var_value(res.group(2), script)
         if not value:
             value = ''
-        return res.group(1) + value + substitute_shell_vars(res.group(3), script)
+        return res.group(1) + value + \
+            substitute_shell_vars(res.group(3), script)
     else:
         return val
 
-def getstatusoutput(cmd, stdoutonly = 0):
+def getstatusoutput(cmd, stdoutonly = False):
     '''A version of commands.getstatusoutput() which can take cmd as a
        sequence, thus making it potentially more secure.'''
     if stdoutonly:
-        proc = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, close_fds=True)
+        proc = subprocess.Popen(cmd, stdin=subprocess.PIPE,
+                                stdout=subprocess.PIPE, close_fds=True)
     else:
-        proc = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
+        proc = subprocess.Popen(cmd, stdin=subprocess.PIPE,
+                                stdout=subprocess.PIPE,
+                                stderr=subprocess.STDOUT, close_fds=True)
     proc.stdin.close()
     text = proc.stdout.read()
     sts = proc.wait()
@@ -78,23 +93,32 @@ def getstatusoutput(cmd, stdoutonly = 0):
     return sts, text
 
 bz2_regex = re.compile('\.t?bz2?$')
+xz_regex = re.compile('\.(t[xl]z|xz|lzma)$')
+
+def catcmd(fname):
+    """Get a 'cat' command that handles possibly compressed files."""
+    cat = 'gzip -dcf'
+    if bz2_regex.search(fname):
+        cat = 'bzip2 -dcf'
+    elif xz_regex.search(fname):
+        cat = 'xz -dc'
+    return cat
 
 # TODO: is_utf8 could probably be implemented natively without iconv...
 
 def is_utf8(fname):
-    cat = 'gzip -dcf'
-    if bz2_regex.search(fname): cat = 'bzip2 -dcf'
-    if fname.endswith('lzma'): cat = 'lzma -dc'
     # TODO: better shell escaping or sequence based command invocation
-    cmd = commands.getstatusoutput('%s "%s" | iconv -f utf-8 -t utf-8 -o /dev/null' % (cat, fname))
+    cmd = commands.getstatusoutput(
+        '%s "%s" | iconv -f utf-8 -t utf-8 -o /dev/null' %
+        (catcmd(fname), fname))
     return not cmd[0]
 
 def is_utf8_str(s):
     try:
         s.decode('UTF-8')
     except:
-        return 0
-    return 1
+        return False
+    return True
 
 def to_utf8(string):
     if string is None:
@@ -134,33 +158,264 @@ def mktemp():
     tmpfile = os.fdopen(tmpfd, 'w')
     return tmpfile, tmpname
 
-def get_default_valid_rpmgroups(filename = ""):
-    """ Get the default rpm group from filename, or from the rpm package if no
-    filename is given"""
+slash_regex = re.compile('/+')
+slashdot_regex = re.compile('/(\.(/|$))+')
+slashend_regex = re.compile('([^/])/+$')
+
+def safe_normpath(path):
+    """Like os.path.normpath but normalizes less aggressively thus being
+    potentially safer for paths containing symlinks."""
+    ret = slash_regex.sub('/', path)
+    ret = slashdot_regex.sub('\\2', ret)
+    ret = slashend_regex.sub('\\1', ret)
+    return ret
+
+def get_default_valid_rpmgroups(filename = None):
+    """Get default rpm groups from filename, or try to look them up from
+    the rpm package (if installed) if no filename is given"""
     groups = []
     if not filename:
-        p = InstalledPkg('rpm')
-        groupsfiles = [x for x in p.files() if x.endswith('/GROUPS')]
-        if groupsfiles:
-            filename = groupsfiles[0]
+        try:
+            p = InstalledPkg('rpm')
+        except:
+            pass
+        else:
+            groupsfiles = [x for x in p.files() if x.endswith('/GROUPS')]
+            if groupsfiles:
+                filename = groupsfiles[0]
     if filename and os.path.exists(filename):
         fobj = open(filename)
         try:
             groups = fobj.read().strip().splitlines()
         finally:
             fobj.close()
-        if not 'Development/Debug' in groups:
+        if 'Development/Debug' not in groups:
             groups.append('Development/Debug')
-            groups.sort()
+        groups.sort()
     return groups
 
+# from yum 3.2.27, rpmUtils.miscutils, with rpmlint modifications
+def compareEVR((e1, v1, r1), (e2, v2, r2)):
+    # return 1: a is newer than b
+    # 0: a and b are the same version
+    # -1: b is newer than a
+    # rpmlint mod: don't stringify None epochs to 'None' strings
+    if e1 is not None:
+        e1 = str(e1)
+    v1 = str(v1)
+    r1 = str(r1)
+    if e2 is not None:
+        e2 = str(e2)
+    v2 = str(v2)
+    r2 = str(r2)
+    rc = rpm.labelCompare((e1, v1, r1), (e2, v2, r2))
+    return rc
+
+# from yum 3.2.27, rpmUtils.miscutils, with rpmlint modifications
+def rangeCompare(reqtuple, provtuple):
+    """returns true if provtuple satisfies reqtuple"""
+    (reqn, reqf, (reqe, reqv, reqr)) = reqtuple
+    (n, f, (e, v, r)) = provtuple
+    if reqn != n:
+        return 0
+
+    # unversioned satisfies everything
+    if not f or not reqf:
+        return 1
+
+    # and you thought we were done having fun
+    # if the requested release is left out then we have
+    # to remove release from the package prco to make sure the match
+    # is a success - ie: if the request is EQ foo 1:3.0.0 and we have 
+    # foo 1:3.0.0-15 then we have to drop the 15 so we can match
+    if reqr is None:
+        r = None
+    # rpmlint mod: don't mess with provided Epoch, doing so breaks e.g.
+    # "Requires: foo < 1.0" should not be satisfied by "Provides: foo = 1:0.5"
+    #if reqe is None:
+    #    e = None
+    if reqv is None: # just for the record if ver is None then we're going to segfault
+        v = None
+
+    # if we just require foo-version, then foo-version-* will match
+    if r is None:
+        reqr = None
+
+    rc = compareEVR((e, v, r), (reqe, reqv, reqr))
+
+    # does not match unless
+    if rc >= 1:
+        if reqf in ['GT', 'GE', 4, 12]:
+            return 1
+        if reqf in ['EQ', 8]:
+            if f in ['LE', 10, 'LT', 2]:
+                return 1                
+        if reqf in ['LE', 'LT', 'EQ', 10, 2, 8]:
+            if f in ['LE', 'LT', 10, 2]:
+                return 1
+
+    if rc == 0:
+        if reqf in ['GT', 4]:
+            if f in ['GT', 'GE', 4, 12]:
+                return 1
+        if reqf in ['GE', 12]:
+            if f in ['GT', 'GE', 'EQ', 'LE', 4, 12, 8, 10]:
+                return 1
+        if reqf in ['EQ', 8]:
+            if f in ['EQ', 'GE', 'LE', 8, 12, 10]:
+                return 1
+        if reqf in ['LE', 10]:
+            if f in ['EQ', 'LE', 'LT', 'GE', 8, 10, 2, 12]:
+                return 1
+        if reqf in ['LT', 2]:
+            if f in ['LE', 'LT', 10, 2]:
+                return 1
+    if rc <= -1:
+        if reqf in ['GT', 'GE', 'EQ', 4, 12, 8]:
+            if f in ['GT', 'GE', 4, 12]:
+                return 1
+        if reqf in ['LE', 'LT', 10, 2]:
+            return 1
+#                if rc >= 1:
+#                    if reqf in ['GT', 'GE', 4, 12]:
+#                        return 1
+#                if rc == 0:
+#                    if reqf in ['GE', 'LE', 'EQ', 8, 10, 12]:
+#                        return 1
+#                if rc <= -1:
+#                    if reqf in ['LT', 'LE', 2, 10]:
+#                        return 1
+
+    return 0
+
+# from yum 3.2.23, rpmUtils.miscutils, with rpmlint modifications
+def formatRequire(name, flags, evr):
+    s = name
+    
+    if flags:
+        if flags & (rpm.RPMSENSE_LESS | rpm.RPMSENSE_GREATER |
+                    rpm.RPMSENSE_EQUAL):
+            s = s + " "
+            if flags & rpm.RPMSENSE_LESS:
+                s = s + "<"
+            if flags & rpm.RPMSENSE_GREATER:
+                s = s + ">"
+            if flags & rpm.RPMSENSE_EQUAL:
+                s = s + "="
+            s = "%s %s" % (s, versionToString(evr))
+    return s
+
+def versionToString(evr):
+    if not isinstance(evr, tuple) and not isinstance(evr, list):
+        # assume string
+        return evr
+    ret = ""
+    if evr[0] is not None and evr[0] != "":
+        ret += str(evr[0]) + ":"
+    ret += evr[1]
+    if evr[2] is not None and evr[2] != "":
+        ret += "-" + evr[2]
+    return ret
+
+# from yum 3.2.23, rpmUtils.miscutils, with some rpmlint modifications
+def stringToVersion(verstring):
+    if verstring in [None, '']:
+        return (None, None, None)
+    epoch = None
+    i = verstring.find(':')
+    if i != -1:
+        try:
+            epoch = str(long(verstring[:i]))
+        except ValueError:
+            # garbage in epoch, ignore it
+            pass
+    j = verstring.find('-')
+    if j != -1:
+        if verstring[i + 1:j] == '':
+            version = None
+        else:
+            version = verstring[i + 1:j]
+        release = verstring[j + 1:]
+    else:
+        if verstring[i + 1:] == '':
+            version = None
+        else:
+            version = verstring[i + 1:]
+        release = None
+    return (epoch, version, release)
+
+def parse_deps(line):
+    '''Parse provides/requires/conflicts/obsoletes line to list of
+       (name, flags, (epoch, version, release)) tuples.'''
+
+    prcos = []
+    tokens = re.split('[\s,]+', line.strip())
+
+    # Drop line continuation backslash in multiline macro definition (for
+    # spec file parsing), e.g.
+    # [...] \
+    # Obsoletes: foo-%1 <= 1.0.0 \
+    # [...] \
+    # (yes, this is an ugly hack and we probably have other problems with
+    #  multiline macro definitions elsewhere...)
+    if tokens[-1] == '\\':
+        del tokens[-1]
+
+    prco = []
+    while tokens:
+        token = tokens.pop(0)
+        if not token:
+            # skip empty tokens
+            continue
+
+        plen = len(prco)
+
+        if plen == 0:
+            prco.append(token)
+
+        elif plen == 1:
+            flags = 0
+            if token[0] in ("=", "<", "<=", ">", ">="):
+                # versioned, flags
+                if "=" in token:
+                    flags |= rpm.RPMSENSE_EQUAL
+                if "<" in token:
+                    flags |= rpm.RPMSENSE_LESS
+                if ">" in token:
+                    flags |= rpm.RPMSENSE_GREATER
+                prco.append(flags)
+            else:
+                # no flags following name, treat as unversioned, add and reset
+                prco.extend((flags, (None, None, None)))
+                prcos.append(tuple(prco))
+                prco = [token]
+
+        elif plen == 2:
+            # last token of versioned one, add and reset
+            prco.append(stringToVersion(token))
+            prcos.append(tuple(prco))
+            prco = []
+
+    plen = len(prco)
+    if plen:
+        if plen == 1:
+            prco.extend((0, (None, None, None)))
+        elif plen == 2:
+            prco.append((None, None, None))
+        prcos.append(tuple(prco))
+
+    return prcos
+
+
 # classes representing package
 
 class Pkg:
 
-    def __init__(self, filename, dirname, header = None, is_source = 0):
+    _magic_from_compressed_re = re.compile('\([^)]+\s+compressed\s+data\\b')
+
+    def __init__(self, filename, dirname, header = None, is_source = False):
         self.filename = filename
-        self.extracted = 0
+        self.extracted = False
         self.dirname = dirname
         self.current_linenum = None
         self._config_files = None
@@ -225,16 +480,18 @@ class Pkg:
     def _extract(self):
         s = os.stat(self.dirname)
         if not stat.S_ISDIR(s[stat.ST_MODE]):
-            sys.stderr.write('unable to access dir %s\n' % self.dirname)
+            warn('Unable to access dir %s' % self.dirname)
             return None
         else:
             self.dirname = tempfile.mkdtemp(
                 prefix = 'rpmlint.%s.' % os.path.basename(self.filename),
                 dir = self.dirname)
             # TODO: better shell escaping or sequence based command invocation
-            command_str = 'rpm2cpio "%s" | (cd "%s"; cpio -id); chmod -R +rX "%s"' % (self.filename, self.dirname, self.dirname)
+            command_str = \
+                'rpm2cpio "%s" | (cd "%s"; cpio -id); chmod -R +rX "%s"' % \
+                (self.filename, self.dirname, self.dirname)
             cmd = commands.getstatusoutput(command_str)
-            self.extracted = 1
+            self.extracted = True
             return cmd
 
     def checkSignature(self):
@@ -259,12 +516,22 @@ class Pkg:
                         ret.append(str(lineno))
                         break
             except Exception, e:
-                printWarning(self, 'read-error', filename, e)
+                Filter.printWarning(self, 'read-error', filename, e)
         finally:
             if in_file:
                 in_file.close()
         return ret
 
+    def langtag(self, tag, lang):
+        """Get value of tag in the given language."""
+        # LANGUAGE trumps other env vars per GNU gettext docs, see also #166
+        orig = os.environ.get('LANGUAGE')
+        os.environ['LANGUAGE'] = lang
+        ret = self[tag]
+        if orig is not None:
+            os.environ['LANGUAGE'] = orig
+        return ret
+
     # return the associative array indexed on file names with
     # the values as: (file perm, file owner, file group, file link to)
     def files(self):
@@ -332,7 +599,8 @@ class Pkg:
         rdevs = self.header[rpm.RPMTAG_FILERDEVS]
         langs = self.header[rpm.RPMTAG_FILELANGS]
         inodes = self.header[rpm.RPMTAG_FILEINODES]
-        deps = self.header[rpm.RPMTAG_FILEREQUIRE]
+        requires = self.header[rpm.RPMTAG_FILEREQUIRE]
+        provides = self.header[rpm.RPMTAG_FILEPROVIDE]
         files = self.header[rpm.RPMTAG_FILENAMES]
         magics = self.header[rpm.RPMTAG_FILECLASS]
 
@@ -352,31 +620,45 @@ class Pkg:
                 pkgfile.mode = modes[idx]
                 pkgfile.user = users[idx]
                 pkgfile.group = groups[idx]
-                pkgfile.linkto = links[idx]
+                pkgfile.linkto = links[idx] and safe_normpath(links[idx])
                 pkgfile.size = sizes[idx]
                 pkgfile.md5 = md5s[idx]
                 pkgfile.mtime = mtimes[idx]
                 pkgfile.rdev = rdevs[idx]
                 pkgfile.inode = inodes[idx]
-                pkgfile.deps = deps[idx]
+                pkgfile.requires = parse_deps(requires[idx])
+                pkgfile.provides = parse_deps(provides[idx])
                 pkgfile.lang = langs[idx]
                 pkgfile.magic = magics[idx]
                 if not pkgfile.magic and _magic:
                     pkgfile.magic = _magic.file(pkgfile.path)
                 if pkgfile.magic is None:
                     pkgfile.magic = ''
+                elif Pkg._magic_from_compressed_re.search(pkgfile.magic):
+                    # Discard magic from inside compressed files ("file -z")
+                    # until PkgFile gets decompression support.  We may get
+                    # such magic strings from package headers already now;
+                    # for example Fedora's rpmbuild as of F-11's 4.7.1 is
+                    # patched so it generates them.
+                    pkgfile.magic = ''
                 self._files[pkgfile.name] = pkgfile
 
     # API to access dependency information
     def obsoletes(self):
+        """Get package Obsoletes as list of
+           (name, flags, (epoch, version, release)) tuples."""
         self._gatherDepInfo()
         return self._obsoletes
 
     def requires(self):
+        """Get package Requires as list of
+           (name, flags, (epoch, version, release)) tuples."""
         self._gatherDepInfo()
         return self._requires
 
     def prereq(self):
+        """Get package PreReqs as list of
+           (name, flags, (epoch, version, release)) tuples."""
         self._gatherDepInfo()
         return self._prereq
 
@@ -386,37 +668,43 @@ class Pkg:
         return self._req_names
 
     def check_versioned_dep(self, name, version):
+        # try to match name%_isa as well (e.g. "foo(x86-64)", "foo(x86-32)")
+        name_re = re.compile('^%s(\(\w+-\d+\))?$' % re.escape(name))
         for d in self.requires() + self.prereq():
-            if d[0] == name:
-                current_version = d[1]
-                if current_version.find(':') > 0:
-                    current_version = ''.join(current_version.split(':')[1:])
-                if d[2] & rpm.RPMSENSE_EQUAL != rpm.RPMSENSE_EQUAL or current_version != version:
-                    return 0
-                else:
-                    return 1
-        return 0
+            if name_re.match(d[0]):
+                if d[1] & rpm.RPMSENSE_EQUAL != rpm.RPMSENSE_EQUAL \
+                        or d[2][1] != version:
+                    return False
+                return True
+        return False
 
     def conflicts(self):
+        """Get package Conflicts as list of
+           (name, flags, (epoch, version, release)) tuples."""
         self._gatherDepInfo()
         return self._conflicts
 
     def provides(self):
+        """Get package Provides as list of
+           (name, flags, (epoch, version, release)) tuples."""
         self._gatherDepInfo()
         return self._provides
 
     # internal function to gather dependency info used by the above ones
-    def _gather_aux(self, header, list, nametag, versiontag, flagstag, prereq = None):
+    def _gather_aux(self, header, list, nametag, flagstag, versiontag,
+                    prereq = None):
         names = header[nametag]
-        versions = header[versiontag]
         flags = header[flagstag]
+        versions = header[versiontag]
 
         if versions:
             for loop in range(len(versions)):
+                evr = stringToVersion(versions[loop])
                 if prereq is not None and flags[loop] & PREREQ_FLAG:
-                    prereq.append((names[loop], versions[loop], flags[loop] & (~PREREQ_FLAG)))
+                    prereq.append((names[loop], flags[loop] & (~PREREQ_FLAG),
+                                   evr))
                 else:
-                    list.append((names[loop], versions[loop], flags[loop]))
+                    list.append((names[loop], flags[loop], evr))
 
     def _gatherDepInfo(self):
         if self._requires is None:
@@ -428,37 +716,36 @@ class Pkg:
 
             self._gather_aux(self.header, self._requires,
                              rpm.RPMTAG_REQUIRENAME,
-                             rpm.RPMTAG_REQUIREVERSION,
                              rpm.RPMTAG_REQUIREFLAGS,
+                             rpm.RPMTAG_REQUIREVERSION,
                              self._prereq)
             self._gather_aux(self.header, self._conflicts,
                              rpm.RPMTAG_CONFLICTNAME,
-                             rpm.RPMTAG_CONFLICTVERSION,
-                             rpm.RPMTAG_CONFLICTFLAGS)
+                             rpm.RPMTAG_CONFLICTFLAGS,
+                             rpm.RPMTAG_CONFLICTVERSION)
             self._gather_aux(self.header, self._provides,
                              rpm.RPMTAG_PROVIDENAME,
-                             rpm.RPMTAG_PROVIDEVERSION,
-                             rpm.RPMTAG_PROVIDEFLAGS)
+                             rpm.RPMTAG_PROVIDEFLAGS,
+                             rpm.RPMTAG_PROVIDEVERSION)
             self._gather_aux(self.header, self._obsoletes,
                              rpm.RPMTAG_OBSOLETENAME,
-                             rpm.RPMTAG_OBSOLETEVERSION,
-                             rpm.RPMTAG_OBSOLETEFLAGS)
+                             rpm.RPMTAG_OBSOLETEFLAGS,
+                             rpm.RPMTAG_OBSOLETEVERSION)
 
 def getInstalledPkgs(name):
     """Get list of installed package objects by name."""
 
     pkgs = []
     ts = rpm.TransactionSet()
-    if re.search('[?*[]', name):
+    if re.search('[?*]|\[.+\]', name):
         mi = ts.dbMatch()
         mi.pattern("name", rpm.RPMMIRE_GLOB, name)
     else:
         mi = ts.dbMatch("name", name)
-    if not mi:
-        raise KeyError(name)
 
     for hdr in mi:
         pkgs.append(InstalledPkg(name, hdr))
+
     return pkgs
 
 # Class to provide an API to an installed package
@@ -476,9 +763,11 @@ class InstalledPkg(Pkg):
 
         Pkg.__init__(self, name, '/', hdr)
 
-        self.extracted = 1
+        self.extracted = True
         # create a fake filename to satisfy some checks on the filename
-        self.filename = '%s-%s-%s.%s.rpm' % (self.name, self[rpm.RPMTAG_VERSION], self[rpm.RPMTAG_RELEASE], self[rpm.RPMTAG_ARCH])
+        self.filename = '%s-%s-%s.%s.rpm' % \
+            (self.name, self[rpm.RPMTAG_VERSION], self[rpm.RPMTAG_RELEASE],
+             self[rpm.RPMTAG_ARCH])
 
     def cleanup(self):
         pass
@@ -513,8 +802,12 @@ class PkgFile(object):
         self.mtime = 0
         self.rdev = ''
         self.inode = 0
-        self.deps = ''
+        self.requires = []
+        self.provides = []
         self.lang = ''
+        self.magic = ''
+
+    # TODO: decompression support
 
     is_config    = property(lambda self: self.flags & rpm.RPMFILE_CONFIG)
     is_doc       = property(lambda self: self.flags & rpm.RPMFILE_DOC)
@@ -526,11 +819,11 @@ class PkgFile(object):
 if __name__ == '__main__':
     for p in sys.argv[1:]:
         pkg = Pkg(sys.argv[1], tempfile.gettempdir())
-        sys.stdout.write('Requires: %s\n' % pkg.requires())
-        sys.stdout.write('Prereq: %s\n' % pkg.prereq())
-        sys.stdout.write('Conflicts: %s\n' % pkg.conflicts())
-        sys.stdout.write('Provides: %s\n' % pkg.provides())
-        sys.stdout.write('Obsoletes: %s\n' % pkg.obsoletes())
+        print ('Requires: %s' % pkg.requires())
+        print ('Prereq: %s' % pkg.prereq())
+        print ('Conflicts: %s' % pkg.conflicts())
+        print ('Provides: %s' % pkg.provides())
+        print ('Obsoletes: %s' % pkg.obsoletes())
         pkg.cleanup()
 
 # Pkg.py ends here
