@@ -3,7 +3,7 @@
 # Project         : Mandriva Linux
 # Module          : rpmlint
 # File            : PostCheck.py
-# Version         : $Id: PostCheck.py 1565 2009-03-17 23:12:41Z scop $
+# Version         : $Id: PostCheck.py 1754 2010-03-30 20:34:48Z scop $
 # Author          : Frederic Lepied
 # Created On      : Wed Jul  5 13:30:17 2000
 # Purpose         : Check post/pre scripts
@@ -43,7 +43,6 @@ home_regex = re.compile('[^a-zA-Z]+~/|\${?HOME(\W|$)', re.MULTILINE)
 dangerous_command_regex = re.compile("(^|[;\|`]|&&|$\()\s*(?:\S*/s?bin/)?(cp|mv|ln|tar|rpm|chmod|chown|rm|cpio|install|perl|userdel|groupdel)\s", re.MULTILINE)
 selinux_regex = re.compile("(^|[;\|`]|&&|$\()\s*(?:\S*/s?bin/)?(chcon|runcon)\s", re.MULTILINE)
 single_command_regex = re.compile("^[ \n]*([^ \n]+)[ \n]*$")
-update_menu_regex = re.compile('update-menus', re.MULTILINE)
 tmp_regex = re.compile('\s(/var)?/tmp', re.MULTILINE)
 menu_regex = re.compile('^/usr/lib/menu/|^/etc/menu-methods/|^/usr/share/applications/')
 bogus_var_regex = re.compile('(\${?RPM_BUILD_(ROOT|DIR)}?)')
@@ -73,7 +72,7 @@ script_tags = [
 
 def incorrect_shell_script(prog, shellscript):
     if not shellscript:
-        return 0
+        return False
     # TODO: test that "prog" is available/executable
     tmpfile, tmpname = Pkg.mktemp()
     try:
@@ -87,7 +86,7 @@ def incorrect_shell_script(prog, shellscript):
 
 def incorrect_perl_script(prog, perlscript):
     if not perlscript:
-        return 0
+        return False
     # TODO: test that "prog" is available/executable
     tmpfile, tmpname = Pkg.mktemp()
     try:
@@ -120,7 +119,8 @@ class PostCheck(AbstractCheck.AbstractCheck):
                 self.check_aux(pkg, files, prog, script, tag, prereq)
             else:
                 for idx in range(0, len(prog)):
-                    self.check_aux(pkg, files, prog[idx], script[idx], tag, prereq)
+                    self.check_aux(
+                        pkg, files, prog[idx], script[idx], tag, prereq)
 
         ghost_files = pkg.ghostFiles()
         if ghost_files:
@@ -130,15 +130,16 @@ class PostCheck(AbstractCheck.AbstractCheck):
                 printWarning(pkg, 'ghost-files-without-postin')
             else:
                 for f in ghost_files:
-                    if (not postin or postin.find(f) == -1) and \
-                       (not prein or prein.find(f) == -1) and \
-                       not f in pkg.missingOkFiles():
-                        printWarning(pkg, 'postin-without-ghost-file-creation', f)
+                    if (not postin or f not in postin) and \
+                       (not prein or f not in prein) and \
+                       f not in pkg.missingOkFiles():
+                        printWarning(pkg,
+                                     'postin-without-ghost-file-creation', f)
 
     def check_aux(self, pkg, files, prog, script, tag, prereq):
         if script:
             if prog:
-                if not prog in valid_shells:
+                if prog not in valid_shells:
                     printError(pkg, 'invalid-shell-in-' + tag[2], prog)
                 if prog in empty_shells:
                     printError(pkg, 'non-empty-' + tag[2], prog)
@@ -149,27 +150,30 @@ class PostCheck(AbstractCheck.AbstractCheck):
                     printWarning(pkg, 'spurious-bracket-in-' + tag[2])
                 res = dangerous_command_regex.search(script)
                 if res:
-                    printWarning(pkg, 'dangerous-command-in-' + tag[2], res.group(2))
+                    printWarning(pkg, 'dangerous-command-in-' + tag[2],
+                                 res.group(2))
                 res = selinux_regex.search(script)
                 if res:
-                    printError(pkg, 'forbidden-selinux-command-in-' + tag[2], res.group(2))
+                    printError(pkg, 'forbidden-selinux-command-in-' + tag[2],
+                               res.group(2))
 
-                if update_menu_regex.search(script):
-                    menu_error = 1
+                if 'update-menus' in script:
+                    menu_error = True
                     for f in files:
                         if menu_regex.search(f):
-                            menu_error = 0
+                            menu_error = False
                             break
                     if menu_error:
-                        printError(pkg, 'update-menus-without-menu-file-in-' + tag[2])
+                        printError(pkg, 'update-menus-without-menu-file-in-' +
+                                   tag[2])
                 if tmp_regex.search(script):
                     printError(pkg, 'use-tmp-in-' + tag[2])
                 for c in prereq_assoc:
                     if c[0].search(script):
-                        found = 0
+                        found = False
                         for p in c[1]:
                             if p in prereq or p in files:
-                                found = 1
+                                found = True
                                 break
                         if not found:
                             printError(pkg, 'no-prereq-on', c[1][0])
@@ -181,7 +185,8 @@ class PostCheck(AbstractCheck.AbstractCheck):
                     printError(pkg, 'use-of-home-in-' + tag[2])
                 res = bogus_var_regex.search(script)
                 if res:
-                    printWarning(pkg, 'bogus-variable-use-in-' + tag[2], res.group(1))
+                    printWarning(pkg, 'bogus-variable-use-in-' + tag[2],
+                                 res.group(1))
 
             if prog == '/usr/bin/perl':
                 if incorrect_perl_script(prog, script):
@@ -190,9 +195,8 @@ class PostCheck(AbstractCheck.AbstractCheck):
             res = single_command_regex.search(script)
             if res:
                 printWarning(pkg, 'one-line-command-in-' + tag[2], res.group(1))
-        else:
-            if prog not in empty_shells and prog in valid_shells:
-                printWarning(pkg, 'empty-' + tag[2])
+        elif prog not in empty_shells and prog in valid_shells:
+            printWarning(pkg, 'empty-' + tag[2])
 
 # Create an object to enable the auto registration of the test
 check = PostCheck()
