@@ -4,20 +4,20 @@
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Thu Oct  7 17:06:14 1999
-# Version       : $Id: SignatureCheck.py 1532 2009-01-30 22:01:50Z scop $
+# Version       : $Id: SignatureCheck.py 1732 2010-02-21 11:28:42Z scop $
 # Purpose       : check the presence of a PGP signature.
 #############################################################################
 
 import re
-import sys
 
 from Filter import addDetails, printError
 import AbstractCheck
+import Pkg
 
 
 class SignatureCheck(AbstractCheck.AbstractCheck):
     pgp_regex = re.compile("pgp|gpg", re.IGNORECASE)
-    unknown_key_regex = re.compile("\(MISSING KEYS:\s+([^\)]+)\)")
+    unknown_key_regex = re.compile("\(MISSING KEYS:(?:\([^)]+\))?\s+([^\)]+)\)")
 
     def __init__(self):
         AbstractCheck.AbstractCheck.__init__(self, "SignatureCheck")
@@ -32,7 +32,8 @@ class SignatureCheck(AbstractCheck.AbstractCheck):
             if kres:
                 printError(pkg, "unknown-key", kres.group(1))
             else:
-                sys.stderr.write("error checking signature of " + pkg.filename + "\n")
+                Pkg.warn("Error checking signature of %s: %s" %
+                         (pkg.filename, res[1]))
         else:
             if not SignatureCheck.pgp_regex.search(res[1]):
                 printError(pkg, "no-signature")
@@ -44,10 +45,10 @@ addDetails(
 'no-signature',
 '''You have to include your pgp or gpg signature in your package.
 For more information on signatures, please refer to www.gnupg.org.''',
+
 'unknown-key',
 '''The package was signed, but with an unknown key.
 See the rpm --import option for more information.''',
-
 )
 
 # SignatureCheck.py ends here
