@@ -4,11 +4,17 @@
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Thu Oct  7 17:06:14 1999
-# Version       : $Id: SpecCheck.py 1568 2009-03-19 21:05:02Z scop $
+# Version       : $Id: SpecCheck.py 1789 2010-06-03 21:30:26Z scop $
 # Purpose       : check the spec file of a source rpm.
 #############################################################################
 
 import re
+try:
+    from urlparse import urlparse
+except ImportError: # Python 3
+    from urllib.parse import urlparse
+
+import rpm
 
 from Filter import addDetails, printError, printWarning
 from TagsCheck import VALID_GROUPS
@@ -24,31 +30,28 @@ DEFAULT_BIARCH_PACKAGES = '^(gcc|glibc)'
 # their noarch files in /usr/lib/<package>/*, or packages that can't
 # be installed on biarch systems
 DEFAULT_HARDCODED_LIB_PATH_EXCEPTIONS = '/lib/(modules|cpp|perl5|rpm|hotplug|firmware)($|[\s/,])'
-patch_regex = re.compile("^Patch(\d*)\s*:\s*([^\s]+)", re.IGNORECASE)
+patch_regex = re.compile("^Patch(\d*)\s*:\s*(\S+)", re.IGNORECASE)
 applied_patch_regex = re.compile("^%patch(\d*)")
 applied_patch_p_regex = re.compile("\s-P\s+(\d+)\\b")
 source_dir_regex = re.compile("^[^#]*(\$RPM_SOURCE_DIR|%{?_sourcedir}?)")
-obsolete_tags_regex = re.compile("^(Copyright|Serial)\s*:\s*([^\s]+)")
-buildroot_regex = re.compile('Buildroot\s*:\s*([^\s]+)', re.IGNORECASE)
-prefix_regex = re.compile('^Prefix\s*:\s*([^\s]+)', re.IGNORECASE)
-packager_regex = re.compile('^Packager\s*:\s*([^\s]+)', re.IGNORECASE)
+obsolete_tags_regex = re.compile("^(Copyright|Serial)\s*:\s*(\S+)")
+buildroot_regex = re.compile('^BuildRoot\s*:\s*(\S+)', re.IGNORECASE)
+prefix_regex = re.compile('^Prefix\s*:\s*(\S+)', re.IGNORECASE)
+packager_regex = re.compile('^Packager\s*:\s*(\S+)', re.IGNORECASE)
 noarch_regex = re.compile('^BuildArch(?:itectures)?\s*:\s*\\bnoarch\\b', re.IGNORECASE)
 make_check_regex = re.compile('(^|\s|%{?__)make}?\s+(check|test)')
 rm_regex = re.compile('(^|\s)((.*/)?rm|%{?__rm}?) ')
-rpm_buildroot_regex = re.compile('(\\\*)\${?RPM_BUILD_ROOT}?|(%+){?buildroot}?')
-configure_start_regex = re.compile('\./configure')
-configure_libdir_spec_regex = re.compile('ln |\./configure[^#]*--libdir=([^\s]+)[^#]*')
+rpm_buildroot_regex = re.compile('^[^#]*(?:(\\\*)\${?RPM_BUILD_ROOT}?|(%+){?buildroot}?)')
+configure_libdir_spec_regex = re.compile('ln |\./configure[^#]*--libdir=(\S+)[^#]*')
 lib_package_regex = re.compile('^%package.*\Wlib')
-mklibname_regex = re.compile('%mklibname')
-ifarch_regex = re.compile('%ifn?arch\s+')
-if_regex = re.compile('%if\s+')
-endif_regex = re.compile('%endif\\b')
+ifarch_regex = re.compile('^\s*%ifn?arch\s')
+if_regex = re.compile('^\s*%if\s')
+endif_regex = re.compile('^\s*%endif\\b')
 biarch_package_regex = re.compile(DEFAULT_BIARCH_PACKAGES)
 hardcoded_lib_path_exceptions_regex = re.compile(Config.getOption('HardcodedLibPathExceptions', DEFAULT_HARDCODED_LIB_PATH_EXCEPTIONS))
 prereq_regex = re.compile('^PreReq(\(.*\))?:\s*(.+?)\s*$', re.IGNORECASE)
 buildprereq_regex = re.compile('^BuildPreReq:\s*(.+?)\s*$', re.IGNORECASE)
 use_utf8 = Config.getOption('UseUTF8', Config.USEUTF8_DEFAULT)
-macro_regex = re.compile('(%+)[{(]?(\w+)')
 libdir_regex = re.compile('%{?_lib(?:dir)?\}?\\b')
 comment_or_empty_regex = re.compile('^\s*(#|$)')
 defattr_regex = re.compile('^\s*%defattr\\b')
@@ -56,7 +59,10 @@ attr_regex = re.compile('^\s*%attr\\b')
 section_regexs = dict(
     ([x, re.compile('^%' + x + '(?:\s|$)')]
      for x in ('build', 'changelog', 'check', 'clean', 'description', 'files',
-               'install', 'package', 'prep')))
+               'install', 'package', 'prep', 'pre', 'post', 'preun', 'postun',
+               'trigger', 'triggerin', 'triggerun', 'triggerprein',
+               'triggerpostun', 'pretrans', 'posttrans')))
+deprecated_grep_regex = re.compile(r'\b[ef]grep\b')
 
 # Only check for /lib, /usr/lib, /usr/X11R6/lib
 # TODO: better handling of X libraries and modules.
@@ -73,8 +79,12 @@ depgen_disable_regex = re.compile('(^|\s)%(define|global)\s+_use_internal_depend
 # See https://bugzilla.redhat.com/488146 for details
 indent_spaces_regex = re.compile('( \t|(^|\t)([^\t]{8})*[^\t]{4}[^\t]?([^\t][^\t.!?]|[^\t]?[.!?] )  )')
 
+requires_regex = re.compile('^(?:Build)?(?:Pre)?Req(?:uires)?(?:\([^\)]+\))?:\s*(.*)', re.IGNORECASE)
 provides_regex = re.compile('^Provides(?:\([^\)]+\))?:\s*(.*)', re.IGNORECASE)
 obsoletes_regex = re.compile('^Obsoletes:\s*(.*)', re.IGNORECASE)
+conflicts_regex = re.compile('^(?:Build)?Conflicts:\s*(.*)', re.IGNORECASE)
+
+compop_regex = re.compile('[<>=]')
 
 setup_q_regex = re.compile(' -[A-Za-z]*q')
 setup_t_regex = re.compile(' -[A-Za-z]*T')
@@ -82,57 +92,14 @@ setup_ab_regex = re.compile(' -[A-Za-z]*[ab]')
 
 filelist_regex = re.compile('\s+-f\s+\S+')
 pkgname_regex = re.compile('\s+(?:-n\s+)?(\S+)')
+tarball_regex = re.compile('\.(?:t(?:ar|[glx]z|bz2?)|zip)\\b', re.IGNORECASE)
 
-def deptokens(line):
-    '''Parse provides/requires/conflicts/obsoletes line to dep token list.'''
-    prco = []
-    tmp = ''
-    wantmore = 0
-    toks = re.split('[\s,]+', line.strip())
 
-    # Drop line continuation backslash in multiline macro definition, eg.
-    # [...] \
-    # Obsoletes: foo-%1 <= 1.0.0 \
-    # [...] \
-    # (yes, this is an ugly hack and we probably have other problems with
-    #  multiline macro definitions elsewhere...)
-    if toks[-1] == '\\':
-        del toks[-1]
-
-    for tok in toks:
-        if len(tok) == 0:
-            continue
-        if len(tmp) == 0:
-            tmp = tok
-        elif wantmore:
-            tmp += ' ' + tok
-            wantmore = 0
-        elif tok[0] in ('=', '<', '>'):
-            tmp += ' ' + tok
-            wantmore = 1
-        else:
-            prco.append(tmp)
-            wantmore = 0
-            tmp = tok
-    if len(tmp) != 0:
-        prco.append(tmp)
-    return prco
-
-def versioned(toks):
-    '''Return versioned dependency tokens from the given list.'''
-    res = []
-    for tok in toks:
-        if tok.find('=') > 0 or tok.find('<') > 0 or tok.find('>') > 0:
-            res.append(tok)
-    return res
-
-def unversioned(toks):
-    '''Return unversioned dependency tokens from the given list.'''
-    res = []
-    for tok in toks:
-        if tok.find(' ') < 0:
-            res.append(tok)
-    return res
+def unversioned(deps):
+    '''Yield unversioned dependency names from the given list.'''
+    for dep in deps:
+        if not dep[1]:
+            yield dep[0]
 
 def contains_buildroot(line):
     '''Check if the given line contains use of rpm buildroot.'''
@@ -140,8 +107,8 @@ def contains_buildroot(line):
     if res and \
            (not res.group(1) or len(res.group(1)) % 2 == 0) and \
            (not res.group(2) or len(res.group(2)) % 2 != 0):
-        return 1
-    return 0
+        return True
+    return False
 
 
 class SpecCheck(AbstractCheck.AbstractCheck):
@@ -161,10 +128,10 @@ class SpecCheck(AbstractCheck.AbstractCheck):
             if fname.endswith('.spec'):
                 self._spec_file = pkgfile.path
                 if fname == pkg.name + ".spec":
-                        wrong_spec = False
-                        break
+                    wrong_spec = False
+                    break
                 else:
-                        wrong_spec = True
+                    wrong_spec = True
         if not self._spec_file:
             printError(pkg, "no-spec-file")
         else:
@@ -174,37 +141,38 @@ class SpecCheck(AbstractCheck.AbstractCheck):
             # check content of spec file
             self.check_spec(pkg, self._spec_file)
 
-    def check_spec(self, pkg, spec_file):
+    def check_spec(self, pkg, spec_file, spec_lines=[]):
         self._spec_file = spec_file
         spec_only = isinstance(pkg, Pkg.FakePkg)
-        spec_lines = Pkg.readlines(spec_file)
+        if not spec_lines:
+            spec_lines = Pkg.readlines(spec_file)
         patches = {}
         applied_patches = []
         applied_patches_ifarch = []
-        source_dir = None
-        buildroot = 0
-        configure = 0
+        source_dir = False
+        buildroot = False
+        configure_linenum = None
         configure_cmdline = ""
-        mklibname = 0
-        lib = 0
+        mklibname = False
+        is_lib_pkg = False
         if_depth = 0
         ifarch_depth = -1
         current_section = 'package'
-        buildroot_clean = {'clean': 0, 'install' : 0}
-        depscript_override = 0
-        depgen_disabled = 0
+        buildroot_clean = {'clean': False, 'install' : False}
+        depscript_override = False
+        depgen_disabled = False
         indent_spaces = 0
         indent_tabs = 0
-        files_has_defattr = 0
+        files_has_defattr = False
         section = {}
         # None == main package
         current_package = None
         package_noarch = {}
 
-        is_utf8 = 0
+        is_utf8 = False
         if self._spec_file and use_utf8:
             if Pkg.is_utf8(self._spec_file):
-                is_utf8 = 1
+                is_utf8 = True
             else:
                 printError(pkg, "non-utf8-spec-file", self._spec_file)
 
@@ -223,15 +191,17 @@ class SpecCheck(AbstractCheck.AbstractCheck):
             if is_utf8:
                 line = unicode(line, "utf-8", "replace")
 
-            if line.find(nbsp) != -1:
-                printWarning(pkg, "non-break-space", "line %s" % pkg.current_linenum)
+            char = line.find(nbsp)
+            if char != -1:
+                printWarning(pkg, "non-break-space", "line %s, char %d" %
+                             (pkg.current_linenum, char))
 
-            section_marker = 0
+            section_marker = False
             for sec, regex in section_regexs.items():
                 res = regex.search(line)
                 if res:
                     current_section = sec
-                    section_marker = 1
+                    section_marker = True
                     section[sec] = section.get(sec, 0) + 1
                     if sec in ('package', 'files'):
                         rest = filelist_regex.sub('', line[res.end()-1:])
@@ -245,23 +215,26 @@ class SpecCheck(AbstractCheck.AbstractCheck):
             if section_marker:
 
                 if current_section == 'files':
-                    files_has_defattr = 0
+                    files_has_defattr = False
 
-                if lib_package_regex.search(line):
-                    lib = 1
+                if not is_lib_pkg and lib_package_regex.search(line):
+                    is_lib_pkg = True
 
                 continue
 
-            if current_section in ('prep', 'build'):
-                if contains_buildroot(line):
-                    printWarning(pkg, 'rpm-buildroot-usage', '%' + current_section, line[:-1].strip())
+            if current_section in ('prep', 'build') and \
+                    contains_buildroot(line):
+                printWarning(pkg, 'rpm-buildroot-usage', '%' + current_section,
+                             line[:-1].strip())
 
-            if make_check_regex.search(line) and current_section not in ('check', 'changelog', 'package', 'description'):
+            if make_check_regex.search(line) and current_section not in \
+                    ('check', 'changelog', 'package', 'description'):
                 printWarning(pkg, 'make-check-outside-check-section', line[:-1])
 
-            if current_section in buildroot_clean:
-                if contains_buildroot(line) and rm_regex.search(line):
-                    buildroot_clean[current_section] = 1
+            if current_section in buildroot_clean and \
+                    not buildroot_clean[current_section] and \
+                    contains_buildroot(line) and rm_regex.search(line):
+                buildroot_clean[current_section] = True
 
             if ifarch_regex.search(line):
                 if_depth = if_depth + 1
@@ -297,10 +270,10 @@ class SpecCheck(AbstractCheck.AbstractCheck):
             elif not source_dir:
                 res = source_dir_regex.search(line)
                 if res:
-                    source_dir = 1
+                    source_dir = True
                     printError(pkg, "use-of-RPM_SOURCE_DIR")
 
-            if configure:
+            if configure_linenum:
                 if configure_cmdline[-1] == "\\":
                     configure_cmdline = configure_cmdline[:-1] + line.strip()
                 else:
@@ -309,28 +282,43 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                         # Hack to get the correct (start of ./configure) line
                         # number displayed:
                         real_linenum = pkg.current_linenum
-                        pkg.current_linenum = configure
+                        pkg.current_linenum = configure_linenum
                         printWarning(pkg, "configure-without-libdir-spec")
                         pkg.current_linenum = real_linenum
                     elif res.group(1):
                         res = re.match(hardcoded_library_paths, res.group(1))
                         if res:
-                            printError(pkg, "hardcoded-library-path", res.group(1), "in configure options")
-                    configure = 0
+                            printError(pkg, "hardcoded-library-path",
+                                       res.group(1), "in configure options")
+                    configure_linenum = None
 
-            if current_section != 'changelog' and configure_start_regex.search(line):
-                configure = pkg.current_linenum # store line where it started
-                configure_cmdline = line.strip()
+            hashPos = line.find("#")
+
+            if current_section != 'changelog':
+                cfgPos = line.find('./configure')
+                if cfgPos != -1 and (hashPos == -1 or hashPos > cfgPos):
+                    # store line where it started
+                    configure_linenum = pkg.current_linenum
+                    configure_cmdline = line.strip()
 
             res = hardcoded_library_path_regex.search(line)
-            if current_section != 'changelog' and res and not (biarch_package_regex.match(pkg.name) or hardcoded_lib_path_exceptions_regex.search(res.group(1).lstrip())):
-                printError(pkg, "hardcoded-library-path", "in", res.group(1).lstrip())
+            if current_section != 'changelog' and res and not \
+                    (biarch_package_regex.match(pkg.name) or
+                     hardcoded_lib_path_exceptions_regex.search(
+                            res.group(1).lstrip())):
+                printError(pkg, "hardcoded-library-path", "in",
+                           res.group(1).lstrip())
 
-            if mklibname_regex.search(line):
-                mklibname = 1
+            if '%mklibname' in line:
+                mklibname = True
 
             if current_section == 'package':
 
+                # Would be cleaner to get sources and patches from the
+                # specfile parsed in Python (see below), but we want to
+                # catch %ifarch'd etc ones as well, and also catch these when
+                # the specfile is not parseable.
+
                 res = patch_regex.search(line)
                 if res:
                     pnum = int(res.group(1) or 0)
@@ -342,9 +330,10 @@ class SpecCheck(AbstractCheck.AbstractCheck):
 
                 res = buildroot_regex.search(line)
                 if res:
-                    buildroot = 1
+                    buildroot = True
                     if res.group(1).startswith('/'):
-                        printWarning(pkg, 'hardcoded-path-in-buildroot-tag', res.group(1))
+                        printWarning(pkg, 'hardcoded-path-in-buildroot-tag',
+                                     res.group(1))
 
                 res = packager_regex.search(line)
                 if res:
@@ -368,27 +357,55 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                     printError(pkg, 'buildprereq-use', res.group(1))
 
                 if scriptlet_requires_regex.search(line):
-                    printError(pkg, 'broken-syntax-in-scriptlet-requires', line.strip())
+                    printError(pkg, 'broken-syntax-in-scriptlet-requires',
+                               line.strip())
+
+                res = requires_regex.search(line)
+                if res:
+                    reqs = Pkg.parse_deps(res.group(1))
+                    for req in unversioned(reqs):
+                        if compop_regex.search(req):
+                            printWarning(pkg, 'comparison-operator-in-deptoken',
+                                         req)
 
                 res = provides_regex.search(line)
                 if res:
-                    for prov in unversioned(deptokens(res.group(1))):
+                    provs = Pkg.parse_deps(res.group(1))
+                    for prov in unversioned(provs):
                         printWarning(pkg, 'unversioned-explicit-provides', prov)
+                        if compop_regex.search(prov):
+                            printWarning(pkg, 'comparison-operator-in-deptoken',
+                                         prov)
 
                 res = obsoletes_regex.search(line)
                 if res:
-                    for obs in unversioned(deptokens(res.group(1))):
+                    obses = Pkg.parse_deps(res.group(1))
+                    for obs in unversioned(obses):
                         printWarning(pkg, 'unversioned-explicit-obsoletes', obs)
+                        if compop_regex.search(obs):
+                            printWarning(pkg, 'comparison-operator-in-deptoken',
+                                         obs)
+
+                res = conflicts_regex.search(line)
+                if res:
+                    confs = Pkg.parse_deps(res.group(1))
+                    for conf in unversioned(confs):
+                        if compop_regex.search(conf):
+                            printWarning(pkg, 'comparison-operator-in-deptoken',
+                                         conf)
 
             if current_section == 'changelog':
-                res = macro_regex.search(line)
-                if res and len(res.group(1)) % 2:
-                    printWarning(pkg, 'macro-in-%changelog', res.group(2))
+                for match in AbstractCheck.macro_regex.findall(line):
+                    res = re.match('%+', match)
+                    if len(res.group(0)) % 2:
+                        printWarning(pkg, 'macro-in-%changelog', match)
             else:
                 if not depscript_override:
-                    depscript_override = depscript_override_regex.search(line)
+                    depscript_override = \
+                        depscript_override_regex.search(line) is not None
                 if not depgen_disabled:
-                    depgen_disabled = depgen_disable_regex.search(line)
+                    depgen_disabled = \
+                        depgen_disable_regex.search(line) is not None
 
             if current_section == 'files':
 
@@ -396,12 +413,14 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                    (ifarch_regex.search(line) or if_regex.search(line) or
                     endif_regex.search(line)):
                     if defattr_regex.search(line):
-                        files_has_defattr = 1;
+                        files_has_defattr = True
                     elif not (files_has_defattr or attr_regex.search(line)):
                         printError(pkg, 'files-attr-not-set')
 
                 # TODO: check scriptlets for these too?
-                if package_noarch.get(current_package) or (current_package not in package_noarch and package_noarch.get(None)):
+                if package_noarch.get(current_package) or \
+                        (current_package not in package_noarch and
+                         package_noarch.get(None)):
                     res = libdir_regex.search(line)
                     if res:
                         pkgname = current_package
@@ -410,11 +429,18 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                         printWarning(pkg, 'libdir-macro-in-noarch-package',
                                      pkgname, line.rstrip())
 
-            if not indent_tabs and line.find('\t') != -1:
+            if not indent_tabs and '\t' in line:
                 indent_tabs = pkg.current_linenum
             if not indent_spaces and indent_spaces_regex.search(line):
                 indent_spaces = pkg.current_linenum
 
+            # Check if egrep or fgrep is used
+            if current_section not in \
+                    ('package', 'changelog', 'description', 'files'):
+                greps = deprecated_grep_regex.findall(line)
+                if greps:
+                    printWarning(pkg, "deprecated-grep", greps)
+
             # If not checking spec file only, we're checking one inside a
             # SRPM -> skip this check to avoid duplicate warnings (#167)
             if spec_only and VALID_GROUPS and \
@@ -423,15 +449,23 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                 if group not in VALID_GROUPS:
                     printWarning(pkg, 'non-standard-group', group)
 
-        # No useful line number info beyond this point.
+            # Test if there are macros in comments
+            if hashPos != -1 and \
+                    (hashPos == 0 or line[hashPos-1] in (" ", "\t")):
+                for match in AbstractCheck.macro_regex.findall(
+                    line[hashPos+1:]):
+                    res = re.match('%+', match)
+                    if len(res.group(0)) % 2:
+                        printWarning(pkg, 'macro-in-comment', match)
+
+        # Last line read is not useful after this point
         pkg.current_linenum = None
 
-        for sect in buildroot_clean:
-            if not buildroot_clean[sect]:
-                printError(pkg, 'no-cleaning-of-buildroot', '%' + sect)
+        for sect in (x for x in buildroot_clean if not buildroot_clean[x]):
+            printWarning(pkg, 'no-cleaning-of-buildroot', '%' + sect)
 
         if not buildroot:
-            printError(pkg, 'no-buildroot-tag')
+            printWarning(pkg, 'no-buildroot-tag')
 
         for sec in ('prep', 'build', 'install', 'clean'):
             if not section.get(sec):
@@ -441,16 +475,18 @@ class SpecCheck(AbstractCheck.AbstractCheck):
             if section.get(sec, 0) > 1:
                 printWarning(pkg, 'more-than-one-%%%s-section' % sec)
 
-        if lib and not mklibname:
+        if is_lib_pkg and not mklibname:
             printError(pkg, 'lib-package-without-%mklibname')
 
         if depscript_override and not depgen_disabled:
             printWarning(pkg, 'depscript-without-disabling-depgen')
 
         if indent_spaces and indent_tabs:
+            pkg.current_linenum = max(indent_spaces, indent_tabs)
             printWarning(pkg, 'mixed-use-of-spaces-and-tabs',
                          '(spaces: line %d, tab: line %d)' %
                          (indent_spaces, indent_tabs))
+            pkg.current_linenum = None
 
         # process gathered info
         for pnum, pfile in patches.items():
@@ -461,13 +497,53 @@ class SpecCheck(AbstractCheck.AbstractCheck):
                 printWarning(pkg, "patch-not-applied", "Patch%d:" % pnum,
                              pfile)
 
+        # Rest of the checks require a real spec file
+        if not self._spec_file:
+            return
+
+        # We'd like to parse the specfile only once using python bindings,
+        # but it seems errors from rpmlib get logged to stderr and we can't
+        # capture and print them nicely, so we do it once each way :P
+
         out = Pkg.getstatusoutput(('env', 'LC_ALL=C', 'rpm', '-q',
                                    '--qf=', '--specfile', self._spec_file))
+        parse_error = False
         for line in out[1].splitlines():
             # No such file or dir hack: https://bugzilla.redhat.com/487855
-            if line.find("No such file or directory") < 0:
+            if 'No such file or directory' not in line:
+                parse_error = True
                 printError(pkg, 'specfile-error', line)
 
+        if not parse_error:
+            # grab sources and patches from parsed spec object to get
+            # them with macros expanded for URL checking
+
+            spec_obj = None
+            try:
+                ts = rpm.TransactionSet()
+                spec_obj = ts.parseSpec(self._spec_file)
+            except:
+                # errors logged above already
+                pass
+            if spec_obj:
+                try:
+                    # rpm < 4.8.0
+                    sources = spec_obj.sources()
+                except TypeError:
+                    # rpm >= 4.8.0
+                    sources = spec_obj.sources
+                for src in sources:
+                    (url, num, flags) = src
+                    (scheme, netloc) = urlparse(url)[0:2]
+                    if flags & 1: # rpmspec.h, rpm.org ticket #123
+                        srctype = "Source"
+                    else:
+                        srctype = "Patch"
+                    tag = '%s%s' % (srctype, num)
+                    if scheme and netloc:
+                        self.check_url(pkg, tag, url)
+                    elif srctype == "Source" and tarball_regex.search(url):
+                        printWarning(pkg, 'invalid-url', '%s:' % tag, url)
 
 # Create an object to enable the auto registration of the test
 check = SpecCheck()
@@ -498,9 +574,16 @@ documentation to see what's wrong.''',
 '''The following tags are obsolete: Copyright and Serial. They must
 be replaced by License and Epoch respectively.''',
 
+'deprecated-grep',
+'''Direct use of grep as egrep or fgrep is deprecated in GNU grep and
+historical in POSIX, use grep -E and grep -F instead.''',
+
 'no-buildroot-tag',
 '''The BuildRoot tag isn't used in your spec. It must be used in order to
-allow building the package as non root on some systems.''',
+allow building the package as non root on some systems. For some rpm versions
+(e.g. rpm.org >= 4.6) the BuildRoot tag is not necessary in specfiles and is
+ignored by rpmbuild; if your package is only going to be built with such rpm
+versions you can ignore this warning.''',
 
 'hardcoded-path-in-buildroot-tag',
 '''A path is hardcoded in your Buildroot tag. It should be replaced
@@ -584,12 +667,15 @@ ways.''',
 unpacking the sources.''',
 
 'no-cleaning-of-buildroot',
-'''You should clean $RPM_BUILD_ROOT in the %clean section and just after the
-beginning of %install section. Use "rm -Rf $RPM_BUILD_ROOT".''',
+'''You should clean $RPM_BUILD_ROOT in the %clean section and in the beginning
+of the %install section. Use "rm -rf $RPM_BUILD_ROOT". Some rpm configurations
+do this automatically; if your package is only going to be built in such
+configurations, you can ignore this warning for the section(s) where your rpm
+takes care of it.''',
 
 'rpm-buildroot-usage',
 '''$RPM_BUILD_ROOT should not be touched during %build or %prep stage, as it
-will break short circuiting.''',
+may break short circuit builds.''',
 
 'make-check-outside-check-section',
 '''Make check or other automated regression test should be run in %check, as
@@ -652,6 +738,16 @@ using %defattr before it in the %files section, or use per line %attr's.''',
 'specfile-error',
 '''This error occurred when rpmlint used rpm to query the specfile.  The error
 is output by rpm and the message should contain more information.''',
+
+'comparison-operator-in-deptoken',
+'''This dependency token contains a comparison operator (<, > or =).  This is
+usually not intended and may be caused by missing whitespace between the token's
+name, the comparison operator and the version string.''',
+
+'macro-in-comment',
+'''There is a unescaped macro after a shell style comment in the specfile.
+Macros are expanded everywhere, so check if it can cause a problem in this
+case and escape the macro with another leading % if appropriate.''',
 )
 
 # SpecCheck.py ends here
