@@ -4,48 +4,64 @@
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Tue Sep 28 00:03:24 1999
-# Version       : $Id: TagsCheck.py 1532 2009-01-30 22:01:50Z scop $
+# Version       : $Id: TagsCheck.py 1791 2010-06-04 21:33:17Z scop $
 # Purpose       : Check a package to see if some rpm tags are present
 #############################################################################
 
 import os
 import re
+try:
+    from urlparse import urlparse
+except ImportError: # Python 3
+    from urllib.parse import urlparse
 
 import rpm
 
-from Filter import addDetails, printError, printWarning
+from Filter import addDetails, printError, printInfo, printWarning
 import AbstractCheck
 import Config
 import FilesCheck
 import Pkg
 
+_use_enchant = Config.getOption("UseEnchant", None)
+if _use_enchant or _use_enchant is None:
+    try:
+        import enchant
+        import enchant.checker
+    except ImportError:
+        enchant = None
+else:
+    enchant = None
+del _use_enchant
 
 DEFAULT_VALID_LICENSES = (
     # OSI approved licenses, http://www.opensource.org/licenses/ (unversioned,
     # trailing "license" dropped based on fuzzy logic, and in well-known cases,
     # the abbreviation used instead of the full name, but list kept sorted by
-    # the full name).  Updated 2009-01-23.
+    # the full name).  Updated 2010-02-01.
     'Academic Free License',
     'Adaptive Public License',
+    'AGPLv3', # Affero GNU Public License
+    'AGPLv3+', # Affero GNU Public License
     'Apache License',
     'Apache Software License',
     'Apple Public Source License',
     'Artistic',
     'Attribution Assurance License',
     'BSD',
+    'Boost Software License',
     'Computer Associates Trusted Open Source License',
     'CDDL', # Common Development and Distribution License
-    'CPL', # Common Public License
+    'Common Public Attribution License',
     'CUA Office Public License',
     'EU DataGrid Software License',
     'Eclipse Public License',
     'Educational Community License',
     'Eiffel Forum License',
     'Entessa Public License',
+    'European Union Public License',
     'Fair License',
     'Frameworx License',
-    'GNU Affero GPLv3',
-    'GNU Affero GPLv3+',
     'GPLv1',
     'GPLv1+',
     'GPLv2',
@@ -58,14 +74,13 @@ DEFAULT_VALID_LICENSES = (
     'LGPLv3+',
     'Historical Permission Notice and Disclaimer',
     'IBM Public License',
-    'Intel Open Source License',
+    'IPA Font License',
     'ISC License',
-    'Jabber Open Source License',
     'Lucent Public License',
     'Microsoft Public License',
     'Microsoft Reciprocal License',
+    'MirOS License',
     'MIT',
-    'CVW License', # MITRE Collaborative Virtual Workspace License
     'Motosoto License',
     'MPL', # Mozilla Public License
     'Multics License',
@@ -76,6 +91,7 @@ DEFAULT_VALID_LICENSES = (
     'Non-profit Open Software License',
     'NTP License',
     'OCLC Research Public License',
+    'OFL', # Open Font License
     'Open Group Test Suite License',
     'Open Software License',
     'PHP License',
@@ -85,8 +101,8 @@ DEFAULT_VALID_LICENSES = (
     'RealNetworks Public Source License',
     'Reciprocal Public License',
     'Ricoh Source Code Public License',
+    'Simple Public License',
     'Sleepycat License',
-    'Sun Industry Standards Source License',
     'Sun Public License',
     'Sybase Open Watcom Public License',
     'University of Illinois/NCSA Open Source License',
@@ -382,7 +398,9 @@ BAD_WORDS = {
 
 DEFAULT_INVALID_REQUIRES = ('^is$', '^not$', '^owned$', '^by$', '^any$', '^package$', '^libsafe\.so\.')
 
-VALID_GROUPS = Config.getOption('ValidGroups', Pkg.get_default_valid_rpmgroups())
+VALID_GROUPS = Config.getOption('ValidGroups', None)
+if VALID_GROUPS is None: # get defaults from rpm package only if it's not set
+    VALID_GROUPS = Pkg.get_default_valid_rpmgroups()
 VALID_LICENSES = Config.getOption('ValidLicenses', DEFAULT_VALID_LICENSES)
 INVALID_REQUIRES = map(re.compile, Config.getOption('InvalidRequires', DEFAULT_INVALID_REQUIRES))
 packager_regex = re.compile(Config.getOption('Packager'))
@@ -390,10 +408,9 @@ changelog_version_regex = re.compile('[^>]([^ >]+)\s*$')
 changelog_text_version_regex = re.compile('^\s*-\s*((\d+:)?[\w\.]+-[\w\.]+)')
 release_ext = Config.getOption('ReleaseExtension')
 extension_regex = release_ext and re.compile(release_ext)
-use_version_in_changelog = Config.getOption('UseVersionInChangelog', 1)
+use_version_in_changelog = Config.getOption('UseVersionInChangelog', True)
 devel_number_regex = re.compile('(.*?)([0-9.]+)(_[0-9.]+)?-devel')
 lib_devel_number_regex = re.compile('^lib(.*?)([0-9.]+)(_[0-9.]+)?-devel')
-url_regex = re.compile('^(ftp|http|https)://')
 invalid_url_regex = re.compile(Config.getOption('InvalidURL'), re.IGNORECASE)
 lib_package_regex = re.compile('(?:^(?:compat-)?lib.*?(\.so.*)?|libs?[\d-]*)$', re.IGNORECASE)
 leading_space_regex = re.compile('^\s+')
@@ -402,41 +419,133 @@ invalid_version_regex = re.compile('([0-9](?:rc|alpha|beta|pre).*)', re.IGNORECA
 # () are here for grouping purpose in the regexp
 forbidden_words_regex = re.compile('(' + Config.getOption('ForbiddenWords') + ')', re.IGNORECASE)
 valid_buildhost_regex = re.compile(Config.getOption('ValidBuildHost'))
-epoch_regex = re.compile('^[0-9]+:')
-use_epoch = Config.getOption('UseEpoch', 0)
+use_epoch = Config.getOption('UseEpoch', False)
 use_utf8 = Config.getOption('UseUTF8', Config.USEUTF8_DEFAULT)
-max_line_len = 79
-tag_regex = re.compile('^((?:Auto(?:Req|Prov|ReqProv)|Build(?:Arch(?:itectures)?|Root)|(?:Build)?Conflicts|(?:Build)?(?:Pre)?Requires|Copyright|(?:CVS|SVN)Id|Dist(?:ribution|Tag|URL)|DocDir|(?:Build)?Enhances|Epoch|Exclu(?:de|sive)(?:Arch|OS)|Group|Icon|License|Name|No(?:Patch|Source)|Obsoletes|Packager|Patch\d*|Prefix(?:es)?|Provides|(?:Build)?Recommends|Release|RHNPlatform|Serial|Source\d*|(?:Build)?Suggests|Summary|(?:Build)?Supplements|URL|Vendor|Version)(?:\([^)]+\))?:)\s*\S', re.IGNORECASE)
+max_line_len = Config.getOption('MaxLineLength', 79)
+tag_regex = re.compile('^((?:Auto(?:Req|Prov|ReqProv)|Build(?:Arch(?:itectures)?|Root)|(?:Build)?Conflicts|(?:Build)?(?:Pre)?Requires|Copyright|(?:CVS|SVN)Id|Dist(?:ribution|Tag|URL)|DocDir|(?:Build)?Enhances|Epoch|Exclu(?:de|sive)(?:Arch|OS)|Group|Icon|License|Name|No(?:Patch|Source)|Obsoletes|Packager|Patch\d*|Prefix(?:es)?|Provides|(?:Build)?Recommends|Release|RHNPlatform|Serial|Source\d*|(?:Build)?Suggests|Summary|(?:Build)?Supplements|(?:Bug)?URL|Vendor|Version)(?:\([^)]+\))?:)\s*\S', re.IGNORECASE)
 punct = '.,:;!?'
+sentence_break_regex = re.compile(r'(^|[.:;!?])\s*$')
+so_dep_regex = re.compile(r'\.so(\.[0-9a-zA-z]+)*(\([^)]*\))*$')
+
+private_so_paths = set()
+for path in ('%perl_archlib', '%perl_vendorarch', '%perl_sitearch',
+             '%python_sitearch', '%ruby_sitearch', '%php_extdir'):
+    epath = rpm.expandMacro(path)
+    if epath != path:
+        private_so_paths.add(epath)
+
+_enchant_checkers = {}
+def spell_check(pkg, str, fmt, lang, ignored):
+
+    dict_found = True
+    warned = set()
+    if enchant:
+        if lang == 'C':
+            lang = 'en_US'
+
+        checker = _enchant_checkers.get(lang)
+        if not checker and lang not in _enchant_checkers:
+            try:
+                checker = enchant.checker.SpellChecker(
+                    lang, filters = [ enchant.tokenize.EmailFilter,
+                                      enchant.tokenize.URLFilter,
+                                      enchant.tokenize.WikiWordFilter ])
+            except enchant.DictNotFoundError:
+                printInfo(pkg, 'enchant-dictionary-not-found', lang)
+                pass
+            _enchant_checkers[lang] = checker
+
+        if checker:
+            # squeeze whitespace to ease leading context check
+            checker.set_text(re.sub(r'\s+', ' ', str))
+            uppername = pkg.name.upper()
+            upperparts = uppername.split('-')
+            if lang.startswith('en'):
+                ups = [x + "'S" for x in upperparts]
+                upperparts.extend(ups)
+            for err in checker:
+
+                # Skip already warned and ignored words
+                if err.word in warned or err.word in ignored:
+                    continue
+
+                # Skip all capitalized words that do not start a sentence
+                if err.word[0].isupper() and not \
+                        sentence_break_regex.search(checker.leading_context(3)):
+                    continue
+
+                upperword = err.word.upper()
+
+                # Skip all uppercase words
+                if err.word == upperword:
+                    continue
+
+                # Skip errors containing package name or equal to a
+                # "component" of it, case insensitively
+                if uppername in upperword or upperword in upperparts:
+                    continue
+
+                # Work around enchant's digit tokenizing behavior:
+                # http://github.com/rfk/pyenchant/issues/issue/3
+                if checker.leading_context(1).isdigit() or \
+                        checker.trailing_context(1).isdigit():
+                    continue
+
+                # Warn and suggest
+                sug = ', '.join(checker.suggest()[:3])
+                if sug:
+                    sug = '-> %s' % sug
+                printWarning(pkg, 'spelling-error', fmt % lang, err.word, sug)
+                warned.add(err.word)
+
+        else:
+            dict_found = False
+
+    if not enchant or not dict_found:
+        for seq in str.split():
+            for word in re.split('[^a-z]+', seq.lower()):
+                if len(word) == 0:
+                    continue
+                correct = BAD_WORDS.get(word)
+                if not correct:
+                    continue
+                if word[0] == '\'':
+                    word = word[1:]
+                if word[-1] == '\'':
+                    word = word[:-1]
+                if word in warned or word in ignored:
+                    continue
+                printWarning(pkg, 'spelling-error', fmt % lang, word, '->',
+                             correct)
+                warned.add(word)
 
-def spell_check(pkg, str, tagname):
-    for seq in str.split():
-        for word in re.split('[^a-z]+', seq.lower()):
-            if len(word) > 0:
-                try:
-                    if word[0] == '\'':
-                        word = word[1:]
-                    if word[-1] == '\'':
-                        word = word[:-1]
-                    correct = BAD_WORDS[word]
-                    printWarning(pkg, 'spelling-error-in-' + tagname, word, correct)
-                except KeyError:
-                    pass
 
 class TagsCheck(AbstractCheck.AbstractCheck):
 
     def __init__(self):
         AbstractCheck.AbstractCheck.__init__(self, 'TagsCheck')
 
+    def _unexpanded_macros(self, pkg, tagname, value, is_url=False):
+        if not value:
+            return
+        for match in AbstractCheck.macro_regex.findall(str(value)):
+            # Do not warn about %XX URL escapes
+            if is_url and re.match('^%[0-9A-F][0-9A-F]$', match, re.I):
+                continue
+            printWarning(pkg, 'unexpanded-macro', tagname, match)
+
     def check(self, pkg):
 
         packager = pkg[rpm.RPMTAG_PACKAGER]
+        self._unexpanded_macros(pkg, 'Packager', packager)
         if not packager:
             printError(pkg, 'no-packager-tag')
-        elif Config.getOption('Packager') and not packager_regex.search(packager):
+        elif Config.getOption('Packager') and \
+                not packager_regex.search(packager):
             printWarning(pkg, 'invalid-packager', packager)
 
         version = pkg[rpm.RPMTAG_VERSION]
+        self._unexpanded_macros(pkg, 'Version', version)
         if not version:
             printError(pkg, 'no-version-tag')
         else:
@@ -445,6 +554,7 @@ class TagsCheck(AbstractCheck.AbstractCheck):
                 printError(pkg, 'invalid-version', version)
 
         release = pkg[rpm.RPMTAG_RELEASE]
+        self._unexpanded_macros(pkg, 'Release', release)
         if not release:
             printError(pkg, 'no-release-tag')
         elif release_ext and not extension_regex.search(release):
@@ -457,26 +567,29 @@ class TagsCheck(AbstractCheck.AbstractCheck):
         else:
             if epoch > 99:
                 printWarning(pkg, 'unreasonable-epoch', epoch)
+            epoch = str(epoch)
 
         if use_epoch:
-            for o in pkg.obsoletes():
-                if o[1] and not epoch_regex.search(o[1]):
-                    printWarning(pkg, 'no-epoch-in-obsoletes', o[0] + ' ' + o[1])
-            for c in pkg.conflicts():
-                if c[1] and not epoch_regex.search(c[1]):
-                    printWarning(pkg, 'no-epoch-in-conflicts', c[0] + ' ' + c[1])
-            for p in pkg.provides():
-                if p[1] and not epoch_regex.search(p[1]):
-                    printWarning(pkg, 'no-epoch-in-provides', p[0] + ' ' + p[1])
+            for o in (x for x in pkg.obsoletes() if x[1] and x[2][0] is None):
+                printWarning(pkg, 'no-epoch-in-obsoletes',
+                             apply(Pkg.formatRequire, o))
+            for c in (x for x in pkg.conflicts() if x[1] and x[2][0] is None):
+                printWarning(pkg, 'no-epoch-in-conflicts',
+                             apply(Pkg.formatRequire, c))
+            for p in (x for x in pkg.provides() if x[1] and x[2][0] is None):
+                printWarning(pkg, 'no-epoch-in-provides',
+                             apply(Pkg.formatRequire, p))
 
         name = pkg.name
         deps = pkg.requires() + pkg.prereq()
-        devel_depend = 0
+        devel_depend = False
         is_devel = FilesCheck.devel_regex.search(name)
         is_source = pkg.isSource()
         for d in deps:
-            if use_epoch and d[1] and d[0][0:7] != 'rpmlib(' and not epoch_regex.search(d[1]):
-                printWarning(pkg, 'no-epoch-in-dependency', d[0] + ' ' + d[1])
+            value = apply(Pkg.formatRequire, d)
+            if use_epoch and d[1] and d[2][0] is None \
+                    and d[0][0:7] != 'rpmlib(':
+                printWarning(pkg, 'no-epoch-in-dependency', value)
             for r in INVALID_REQUIRES:
                 if r.search(d[0]):
                     printError(pkg, 'invalid-dependency', d[0])
@@ -484,50 +597,56 @@ class TagsCheck(AbstractCheck.AbstractCheck):
             if d[0].startswith('/usr/local/'):
                 printError(pkg, 'invalid-dependency', d[0])
 
-            if not devel_depend and not is_devel and not is_source:
-                if FilesCheck.devel_regex.search(d[0]):
-                    printError(pkg, 'devel-dependency', d[0])
-                    devel_depend = 1
+            if not devel_depend and not is_devel and not is_source and \
+                    FilesCheck.devel_regex.search(d[0]):
+                printError(pkg, 'devel-dependency', d[0])
+                devel_depend = True
             if is_source and lib_devel_number_regex.search(d[0]):
                 printError(pkg, 'invalid-build-requires', d[0])
             if not is_source and not is_devel:
                 res = lib_package_regex.search(d[0])
                 if res and not res.group(1) and not d[1]:
                     printError(pkg, 'explicit-lib-dependency', d[0])
-            if d[2] == rpm.RPMSENSE_EQUAL and d[1].find('-') != -1:
-                printWarning(pkg, 'requires-on-release', d[0], d[1])
-            if d[1].find('%') != -1:
-                printError(pkg, 'percent-in-dependency', d[0], d[1])
+            if d[1] == rpm.RPMSENSE_EQUAL and d[2][2] is not None:
+                printWarning(pkg, 'requires-on-release', value)
+            self._unexpanded_macros(pkg, 'dependency %s' % (value,), value)
 
+        self._unexpanded_macros(pkg, 'Name', name)
         if not name:
             printError(pkg, 'no-name-tag')
         else:
             if is_devel and not is_source:
                 base = is_devel.group(1)
                 dep = None
-                has_so = 0
+                has_so = False
                 for fname in pkg.files():
                     if fname.endswith('.so'):
-                        has_so = 1
+                        has_so = True
                         break
                 if has_so:
                     base_or_libs = base + '/' + base + '-libs/lib' + base
+                    # try to match *%_isa as well (e.g. "(x86-64)", "(x86-32)")
+                    base_or_libs_re = re.compile(
+                        '^(lib)?%s(-libs)?(\(\w+-\d+\))?$' % re.escape(base))
                     for d in deps:
-                        if d[0] == base or d[0] == base + '-libs' or d[0] == 'lib' + base:
+                        if base_or_libs_re.match(d[0]):
                             dep = d
                             break
                     if not dep:
                         printWarning(pkg, 'no-dependency-on', base_or_libs)
                     elif version:
-                        if epoch is not None: # regardless of use_epoch
-                            expected = str(epoch) + ":" + version
-                        else:
-                            expected = version
-                        if dep[1][:len(expected)] != expected:
-                            if dep[1] != '':
-                                printWarning(pkg, 'incoherent-version-dependency-on', base_or_libs, dep[1], expected)
-                            else:
-                                printWarning(pkg, 'no-version-dependency-on', base_or_libs, expected)
+                        exp = (epoch, version, None)
+                        sexp = Pkg.versionToString(exp)
+                        if not dep[1]:
+                            printWarning(pkg, 'no-version-dependency-on',
+                                         base_or_libs, sexp)
+                        elif dep[2][:2] != exp[:2]:
+                            printWarning(pkg,
+                                         'incoherent-version-dependency-on',
+                                         base_or_libs,
+                                         Pkg.versionToString(dep[2][0],
+                                                             dep[2][1], None),
+                                         sexp)
                     res = devel_number_regex.search(name)
                     if not res:
                         printWarning(pkg, 'no-major-in-name', name)
@@ -540,70 +659,49 @@ class TagsCheck(AbstractCheck.AbstractCheck):
                         if prov not in (x[0] for x in pkg.provides()):
                             printWarning(pkg, 'no-provides', prov)
 
+        # List of words to ignore in spell check
+        ignored_words = set()
+        for pf in pkg.files():
+            ignored_words.update(pf.split('/'))
+        ignored_words.update((x[0] for x in pkg.provides()))
+        ignored_words.update((x[0] for x in pkg.requires()))
+        ignored_words.update((x[0] for x in pkg.conflicts()))
+        ignored_words.update((x[0] for x in pkg.obsoletes()))
+
         summary = pkg[rpm.RPMTAG_SUMMARY]
         if not summary:
             printError(pkg, 'no-summary-tag')
         else:
-            utf8summary = summary
-            if use_utf8:
-                utf8summary = Pkg.to_utf8(summary).decode('utf-8')
-            spell_check(pkg, summary, 'summary')
-            if summary.find('\n') != -1:
-                printError(pkg, 'summary-on-multiple-lines')
-            if summary[0] != summary[0].upper():
-                printWarning(pkg, 'summary-not-capitalized', summary)
-            if summary[-1] == '.':
-                printWarning(pkg, 'summary-ended-with-dot', summary)
-            if len(utf8summary) > max_line_len:
-                printError(pkg, 'summary-too-long', summary)
-            if leading_space_regex.search(summary):
-                printError(pkg, 'summary-has-leading-spaces', summary)
-            res = forbidden_words_regex.search(summary)
-            if res and Config.getOption('ForbiddenWords'):
-                printWarning(pkg, 'summary-use-invalid-word', res.group(1))
-            if name:
-                sepchars = '[\s' + punct + ']'
-                res = re.search('(?:^|\s)(%s)(?:%s|$)' %
-                                (re.escape(name), sepchars),
-                                summary, re.IGNORECASE | re.UNICODE)
-                if res:
-                    printWarning(pkg, 'name-repeated-in-summary', res.group(1))
-            if use_utf8 and not Pkg.is_utf8_str(summary):
-                printError(pkg, 'tag-not-utf8', 'Summary')
+            if not pkg[rpm.RPMTAG_HEADERI18NTABLE]:
+                self._unexpanded_macros(pkg, 'Summary', summary)
+            else:
+                for lang in pkg[rpm.RPMTAG_HEADERI18NTABLE]:
+                    self.check_summary(pkg, lang, ignored_words)
 
         description = pkg[rpm.RPMTAG_DESCRIPTION]
         if not description:
             printError(pkg, 'no-description-tag')
         else:
-            spell_check(pkg, description, 'description')
-            for l in description.splitlines():
-                utf8l = l
-                if use_utf8:
-                    utf8l = Pkg.to_utf8(l).decode('utf-8')
-                if len(utf8l) > max_line_len:
-                    printError(pkg, 'description-line-too-long', l)
-                res = forbidden_words_regex.search(l)
-                if res and Config.getOption('ForbiddenWords'):
-                    printWarning(pkg, 'description-use-invalid-word', res.group(1))
-                res = tag_regex.search(l)
-                if res:
-                    printWarning(pkg, 'tag-in-description', res.group(1))
-            if use_utf8 and not Pkg.is_utf8_str(description):
-                printError(pkg, 'tag-not-utf8', '%description')
+            if not pkg[rpm.RPMTAG_HEADERI18NTABLE]:
+                self._unexpanded_macros(pkg, '%description', description)
+            else:
+                for lang in pkg[rpm.RPMTAG_HEADERI18NTABLE]:
+                    self.check_description(pkg, lang, ignored_words)
 
         group = pkg[rpm.RPMTAG_GROUP]
+        self._unexpanded_macros(pkg, 'Group', group)
         if not group:
             printError(pkg, 'no-group-tag')
-        else:
-            if VALID_GROUPS and group not in VALID_GROUPS:
-                printWarning(pkg, 'non-standard-group', group)
+        elif VALID_GROUPS and group not in VALID_GROUPS:
+            printWarning(pkg, 'non-standard-group', group)
 
         buildhost = pkg[rpm.RPMTAG_BUILDHOST]
+        self._unexpanded_macros(pkg, 'BuildHost', buildhost)
         if not buildhost:
             printError(pkg, 'no-buildhost-tag')
-        else:
-            if Config.getOption('ValidBuildHost') and not valid_buildhost_regex.search(buildhost):
-                printWarning(pkg, 'invalid-buildhost', buildhost)
+        elif Config.getOption('ValidBuildHost') and \
+                not valid_buildhost_regex.search(buildhost):
+            printWarning(pkg, 'invalid-buildhost', buildhost)
 
         changelog = pkg[rpm.RPMTAG_CHANGELOGNAME]
         if not changelog:
@@ -628,17 +726,18 @@ class TagsCheck(AbstractCheck.AbstractCheck):
                         # Allow EVR in changelog without release extension,
                         # the extension is often a macro or otherwise dynamic.
                         if release_ext:
-                            expected.append(extension_regex.sub('', expected[0]))
+                            expected.append(
+                                extension_regex.sub('', expected[0]))
                         if ret.group(1) not in expected:
                             if len(expected) == 1: expected = expected[0]
-                            printWarning(pkg, 'incoherent-version-in-changelog', ret.group(1), expected)
+                            printWarning(pkg, 'incoherent-version-in-changelog',
+                                         ret.group(1), expected)
 
             if clt: changelog = changelog + clt
             if use_utf8 and not Pkg.is_utf8_str(' '.join(changelog)):
                 printError(pkg, 'tag-not-utf8', '%changelog')
 
-#         provides = pkg.provides()
-#         for (provide_name, provide_version, provide_flags) in provides:
+#         for provide_name in (x[0] for x in pkg.provides()):
 #             if name == provide_name:
 #                 printWarning(pkg, 'package-provides-itself')
 #                 break
@@ -651,6 +750,7 @@ class TagsCheck(AbstractCheck.AbstractCheck):
         if not rpm_license:
             printError(pkg, 'no-license')
         else:
+            valid_license = True
             if rpm_license not in VALID_LICENSES:
                 for l1 in split_license(rpm_license):
                     if l1 in VALID_LICENSES:
@@ -658,51 +758,139 @@ class TagsCheck(AbstractCheck.AbstractCheck):
                     for l2 in split_license(l1):
                         if l2 not in VALID_LICENSES:
                             printWarning(pkg, 'invalid-license', l2)
+                            valid_license = False
+            if not valid_license:
+                self._unexpanded_macros(pkg, 'License', rpm_license)
 
-        url = pkg[rpm.RPMTAG_URL]
-        if url and url != 'none':
-            if not url_regex.search(url):
-                printWarning(pkg, 'invalid-url', url)
-            elif Config.getOption('InvalidURL') and invalid_url_regex.search(url):
-                printWarning(pkg, 'invalid-url', url)
-        else:
-            printWarning(pkg, 'no-url-tag')
+        for tag in ('URL', 'DistURL', 'BugURL'):
+            if hasattr(rpm, 'RPMTAG_%s' % tag.upper()):
+                url = pkg[getattr(rpm, 'RPMTAG_%s' % tag.upper())]
+                self._unexpanded_macros(pkg, tag, url, is_url = True)
+                if url:
+                    (scheme, netloc) = urlparse(url)[0:2]
+                    if not scheme or not netloc or "." not in netloc or \
+                            scheme not in ('http', 'https', 'ftp') or \
+                            (Config.getOption('InvalidURL') and \
+                             invalid_url_regex.search(url)):
+                        printWarning(pkg, 'invalid-url', tag, url)
+                    else:
+                        self.check_url(pkg, tag, url)
+                elif tag == 'URL':
+                    printWarning(pkg, 'no-url-tag')
 
         obs_names = [x[0] for x in pkg.obsoletes()]
         prov_names = [x[0] for x in pkg.provides()]
 
-        if pkg.name in obs_names:
-            printError(pkg, 'obsolete-on-name')
-        for o in obs_names:
-            if not o in prov_names:
-                printWarning(pkg, 'obsolete-not-provided', o)
+        for o in (x for x in obs_names if x not in prov_names):
+            printWarning(pkg, 'obsolete-not-provided', o)
         for o in pkg.obsoletes():
-            if o[1].find('%') != -1:
-                printError(pkg, 'percent-in-obsoletes', o[0], o[1])
+            value = apply(Pkg.formatRequire, o)
+            self._unexpanded_macros(pkg, 'Obsoletes %s' % (value,), value)
 
         # TODO: should take versions, <, <=, =, >=, > into account here
         #       https://bugzilla.redhat.com/460872
         useless_provides = []
         for p in prov_names:
-            if prov_names.count(p) != 1:
-                if p not in useless_provides:
-                    useless_provides.append(p)
+            if prov_names.count(p) != 1 and p not in useless_provides:
+                useless_provides.append(p)
         for p in useless_provides:
             printError(pkg, 'useless-provides', p)
 
         for p in pkg.provides():
-            if p[1].find('%') != -1:
-                printError(pkg, 'percent-in-provides', p[0], p[1])
+            value = apply(Pkg.formatRequire, p)
+            self._unexpanded_macros(pkg, 'Provides %s' % (value,), value)
 
         for c in pkg.conflicts():
-            if c[1].find('%') != -1:
-                printError(pkg, 'percent-in-conflicts', c[0], c[1])
+            value = apply(Pkg.formatRequire, c)
+            self._unexpanded_macros(pkg, 'Conflicts %s' % (value,), value)
 
-        expected = '%s-%s-%s.%s.rpm' % (name, version, release, pkg.arch)
+        obss = pkg.obsoletes()
+        if obss:
+            provs = pkg.provides()
+            for prov in provs:
+                for obs in obss:
+                    if Pkg.rangeCompare(obs, prov):
+                        printWarning(pkg, 'self-obsoletion', '%s obsoletes %s' %
+                                     (apply(Pkg.formatRequire, obs),
+                                      apply(Pkg.formatRequire, prov)))
+
+        expfmt = rpm.expandMacro("%{_build_name_fmt}")
+        if pkg.isSource():
+            # _build_name_fmt often (always?) ends up not outputting src/nosrc
+            # as arch for source packages, do it ourselves
+            expfmt = re.sub(r'(?i)%\{?ARCH\b\}?', pkg.arch, expfmt)
+        expected = pkg.header.sprintf(expfmt).split("/")[-1]
         basename = os.path.basename(pkg.filename)
         if basename != expected:
             printWarning(pkg, 'non-coherent-filename', basename, expected)
 
+        for tag in ('Distribution', 'DistTag', 'ExcludeArch', 'ExcludeOS',
+                    'Vendor'):
+            if hasattr(rpm, 'RPMTAG_%s' % tag.upper()):
+                self._unexpanded_macros(
+                    pkg, tag, pkg[getattr(rpm, 'RPMTAG_%s' % tag.upper())])
+
+        for path in private_so_paths:
+            for fname, pkgfile in pkg.files().items():
+                if fname.startswith(path):
+                    for prov in pkgfile.provides:
+                        if so_dep_regex.search(prov[0]):
+                            printWarning(pkg, "private-shared-object-provides",
+                                         fname, apply(Pkg.formatRequire, prov))
+
+
+    def check_description(self, pkg, lang, ignored_words):
+        description = pkg.langtag(rpm.RPMTAG_DESCRIPTION, lang)
+        self._unexpanded_macros(pkg, '%%description -l %s' % lang, description)
+        utf8desc = description
+        if use_utf8:
+            utf8desc = Pkg.to_utf8(description).decode('utf-8')
+        spell_check(pkg, utf8desc, '%%description -l %s', lang, ignored_words)
+        for l in utf8desc.splitlines():
+            if len(l) > max_line_len:
+                printError(pkg, 'description-line-too-long', lang, l)
+            res = forbidden_words_regex.search(l)
+            if res and Config.getOption('ForbiddenWords'):
+                printWarning(pkg, 'description-use-invalid-word', lang,
+                             res.group(1))
+            res = tag_regex.search(l)
+            if res:
+                printWarning(pkg, 'tag-in-description', lang, res.group(1))
+        if use_utf8 and not Pkg.is_utf8_str(description):
+            printError(pkg, 'tag-not-utf8', '%description', lang)
+
+    def check_summary(self, pkg, lang, ignored_words):
+        summary = pkg.langtag(rpm.RPMTAG_SUMMARY, lang)
+        self._unexpanded_macros(pkg, 'Summary(%s)' % lang, summary)
+        utf8summary = summary
+        if use_utf8:
+            utf8summary = Pkg.to_utf8(summary).decode('utf-8')
+        spell_check(pkg, utf8summary, 'Summary(%s)', lang, ignored_words)
+        if '\n' in summary:
+            printError(pkg, 'summary-on-multiple-lines', lang)
+        if summary[0] != summary[0].upper():
+            printWarning(pkg, 'summary-not-capitalized', lang, summary)
+        if summary[-1] == '.':
+            printWarning(pkg, 'summary-ended-with-dot', lang, summary)
+        if len(utf8summary) > max_line_len:
+            printError(pkg, 'summary-too-long', lang, summary)
+        if leading_space_regex.search(summary):
+            printError(pkg, 'summary-has-leading-spaces', lang, summary)
+        res = forbidden_words_regex.search(summary)
+        if res and Config.getOption('ForbiddenWords'):
+            printWarning(pkg, 'summary-use-invalid-word', lang, res.group(1))
+        if pkg.name:
+            sepchars = '[\s' + punct + ']'
+            res = re.search('(?:^|\s)(%s)(?:%s|$)' %
+                            (re.escape(pkg.name), sepchars),
+                            summary, re.IGNORECASE | re.UNICODE)
+            if res:
+                printWarning(pkg, 'name-repeated-in-summary', lang,
+                             res.group(1))
+        if use_utf8 and not Pkg.is_utf8_str(summary):
+            printError(pkg, 'tag-not-utf8', 'Summary', lang)
+
+
 # Create an object to enable the auto registration of the test
 check = TagsCheck()
 
@@ -717,19 +905,16 @@ because when the final version will be out, you will have to use an Epoch tag
 to make the package upgradable. Instead put it in the release tag, prefixed
 with something you have control over.''',
 
-'spelling-error-in-description',
-'''You made a misspelling in the Description. Please double-check.''',
-
-'spelling-error-in-summary',
-'''You made a misspelling in the Summary. Please double-check.''',
+'spelling-error',
+'''The value of this tag appears to be misspelled. Please double-check.''',
 
 'no-packager-tag',
-'''There is no Packager tag in your package. You have to specify a packager using
-the Packager tag. Ex: Packager: John Doo <john.doo@example.com>.''',
+'''There is no Packager tag in your package. You have to specify a packager
+using the Packager tag. Ex: Packager: John Doe <john.doe@example.com>.''',
 
 'invalid-packager',
-'''The packager email must finish with a email compatible with the Packager option
-of rpmlint. Please change it and rebuild your package.''',
+'''The packager email must end with an email compatible with the Packager
+option of rpmlint. Please change it and rebuild your package.''',
 
 'no-version-tag',
 '''There is no Version tag in your package. You have to specify a version using
@@ -767,8 +952,8 @@ Name tag.''',
 ''',
 
 'no-provides',
-'''Your library package doesn't provide the -devel name without the major version
-included.''',
+'''Your library package doesn't provide the -devel name without the major
+version included.''',
 
 'no-summary-tag',
 '''There is no Summary tag in your package. You have to describe your package
@@ -832,10 +1017,6 @@ your specfile.''',
 '''The value of the License tag was not recognized.  Known values are:
 "%s".''' % '", "'.join(VALID_LICENSES),
 
-'invalid-url',
-'''Your URL is not valid. It must begin with http, https or ftp and must no
-longer contain the word mandrake.''',
-
 'obsolete-not-provided',
 '''If a package is obsoleted by a compatible replacement, the obsoleted package
 should also be provided in order to not cause unnecessary dependency breakage.
@@ -870,8 +1051,8 @@ Epoch tag.''',
 package itself.''',
 
 'invalid-build-requires',
-'''Your source package contains a dependency not compliant with the lib64 naming.
-This BuildRequires dependency will not be resolved on lib64 platforms
+'''Your source package contains a dependency not compliant with the lib64
+naming. This BuildRequires dependency will not be resolved on lib64 platforms
 (eg. amd64).''',
 
 'explicit-lib-dependency',
@@ -882,31 +1063,12 @@ explicit Requires: tags.''',
 '''This package provides 2 times the same capacity. It should only provide it
 once.''',
 
-'obsolete-on-name',
-'''A package should not obsolete itself, as it can cause weird errors in tools.''',
-
 'tag-not-utf8',
 '''The character encoding of the value of this tag is not UTF-8.''',
 
 'requires-on-release',
 '''This rpm requires a specific release of another package.''',
 
-'percent-in-dependency',
-'''This rpm has a dependency whose version part contains the '%' character.
-It could be an unexpanded macro, please double check.''',
-
-'percent-in-obsoletes',
-'''This rpm has an Obsoletes whose version part contains the '%' character.
-It could be an unexpanded macro, please double check.''',
-
-'percent-in-provides',
-'''This rpm has a Provides whose version part contains the '%' character.
-It could be an unexpanded macro, please double check.''',
-
-'percent-in-conflicts',
-'''This rpm has a Conflicts whose version part contains the '%' character.
-It could be an unexpanded macro, please double check.''',
-
 'no-url-tag',
 '''The URL tag is missing.''',
 
@@ -914,6 +1076,31 @@ It could be an unexpanded macro, please double check.''',
 '''The name of the package is repeated in its summary.  This is often redundant
 information and looks silly in various programs' output.  Make the summary
 brief and to the point without including redundant information in it.''',
+
+'enchant-dictionary-not-found',
+'''A dictionary for the Enchant spell checking library is not available for
+the language given in the info message.  Spell checking will proceed with
+rpmlint's built-in implementation for localized tags in this language.
+For better spell checking results in this language, install the appropriate
+dictionary that Enchant will use for this language, often for example
+hunspell-* or aspell-*.''',
+
+'self-obsoletion',
+'''The package obsoletes itself.  This is known to cause errors in various
+tools and should thus be avoided, usually by using appropriately versioned
+Obsoletes and/or Provides and avoiding unversioned ones.''',
+
+'unexpanded-macro',
+'''This tag contains something that looks like an unexpanded macro; this is
+often the sign of a misspelling. Please check your specfile.''',
+
+'private-shared-object-provides',
+'''A shared object soname provides is provided by a file in a path from which
+other packages should not directly load shared objects from.  Such shared
+objects should thus not be depended on and they should not result in provides
+in the containing package.  Get rid of the provides if appropriate, for example
+by filtering it out during build.  Note that in some cases this may require
+disabling rpmbuild's internal dependency generator.''',
 )
 
 # TagsCheck.py ends here
