@@ -4,7 +4,7 @@
 # Package       : rpmlint
 # Author        : Frederic Lepied
 # Created on    : Mon Sep 27 19:20:18 1999
-# Version       : $Id: rpmlint.py 1573 2009-03-19 22:47:01Z scop $
+# Version       : $Id: rpmlint.py 1760 2010-04-01 19:26:49Z scop $
 # Purpose       : main entry point: process options, load the checks and run
 #                 the checks.
 #############################################################################
@@ -12,13 +12,12 @@
 import getopt
 import glob
 import imp
+import locale
 import os
 import stat
 import sys
 import tempfile
 
-import rpm
-
 # Do not import anything that initializes its global variables from
 # Config at load time here (or anything that imports such a thing),
 # that results in those variables initialized before config files are
@@ -32,18 +31,30 @@ import Config
 import Pkg
 
 
-version = '@VERSION@'
+_default_user_conf = '%s/rpmlint' % \
+    (os.environ.get('XDG_CONFIG_HOME') or '~/.config')
 
 # Print usage information
 def usage(name):
-    print 'usage:', name, \
-          '[<options>] <rpm files|installed packages|specfiles|dirs>'
-    print '  options in:'
-    print '\t[-i|--info]\n\t[-I <error,error,...>]\n\t[-c|--check <check>]\n\t[-a|--all]\n\t[-C|--checkdir <checkdir>]\n\t[-h|--help]\n\t[-v|--verbose]\n\t[-E|--extractdir <dir>]\n\t[-V|--version]\n\t[-n|--noexception]\n\t[-f|--file <config file to use instead of ~/.rpmlintrc>]'
+    print ('''usage: %s [<options>] <rpm files|installed packages|specfiles|dirs>
+  options:
+\t[-i|--info]
+\t[-I <messageid,messageid,...>]
+\t[-c|--check <check>]
+\t[-a|--all]
+\t[-C|--checkdir <checkdir>]
+\t[-h|--help]
+\t[-v|--verbose]
+\t[-E|--extractdir <dir>]
+\t[-V|--version]
+\t[-n|--noexception]
+\t[-f|--file <user config file to use instead of %s]
+\t[-o|--option <key value>]''' \
+        % (name, _default_user_conf))
 
 # Print version information
 def printVersion():
-    print 'rpmlint version', version, 'Copyright (C) 1999-2007 Frederic Lepied, Mandriva'
+    print ('rpmlint version %s Copyright (C) 1999-2007 Frederic Lepied, Mandriva' % Config.__version__)
 
 def loadCheck(name):
     '''Load a (check) module by its name, unless it is already loaded.'''
@@ -62,6 +73,8 @@ def loadCheck(name):
 #############################################################################
 def main():
 
+    locale.setlocale(locale.LC_COLLATE, '')
+
     # Add check dirs to the front of load path
     sys.path[0:0] = Config.checkDirs()
 
@@ -80,45 +93,64 @@ def main():
     try:
         # Loop over all file names given in arguments
         dirs = []
-        for f in args:
+        for arg in args:
             pkgs = []
             isfile = False
             try:
+                if arg == "-":
+                    arg = "(standard input)"
+                    # Short-circuit stdin spec file check
+                    if do_spec_check:
+                        stdin = sys.stdin.readlines()
+                        if not stdin:
+                            continue
+                        pkg = Pkg.FakePkg(arg)
+                        check = SpecCheck.SpecCheck()
+                        check.verbose = verbose
+                        check.check_spec(pkg, None, spec_lines=stdin)
+                        pkg.cleanup()
+                        specfiles_checked += 1
+                    continue
+
                 try:
-                    st = os.stat(f)
+                    st = os.stat(arg)
                     isfile = True
                     if stat.S_ISREG(st[stat.ST_MODE]):
-                        if not f.endswith(".spec"):
-                            pkgs.append(Pkg.Pkg(f, extract_dir))
+                        if not arg.endswith(".spec"):
+                            pkgs.append(Pkg.Pkg(arg, extract_dir))
                         elif do_spec_check:
                             # Short-circuit spec file checks
-                            pkg = Pkg.FakePkg(f)
+                            pkg = Pkg.FakePkg(arg)
                             check = SpecCheck.SpecCheck()
-                            check.check_spec(pkg, f)
+                            check.verbose = verbose
+                            check.check_spec(pkg, arg)
                             pkg.cleanup()
                             specfiles_checked += 1
 
                     elif stat.S_ISDIR(st[stat.ST_MODE]):
-                        dirs.append(f)
+                        dirs.append(arg)
                         continue
                     else:
                         raise OSError
                 except OSError:
-                    ipkgs = Pkg.getInstalledPkgs(f)
+                    ipkgs = Pkg.getInstalledPkgs(arg)
                     if not ipkgs:
-                        sys.stderr.write(
-                            '(none): E: no installed packages by name %s\n' % f)
+                        Pkg.warn(
+                            '(none): E: no installed packages by name %s' % arg)
                     else:
+                        ipkgs.sort(key = lambda x: locale.strxfrm(
+                                x.header.sprintf("%{NAME}.%{ARCH}")))
                         pkgs.extend(ipkgs)
             except KeyboardInterrupt:
                 if isfile:
-                    f = os.path.abspath(f)
-                sys.stderr.write('(none): E: interrupted, exiting while reading %s\n' % f)
+                    arg = os.path.abspath(arg)
+                Pkg.warn(
+                    '(none): E: interrupted, exiting while reading %s' % arg)
                 sys.exit(2)
             except Exception, e:
                 if isfile:
-                    f = os.path.abspath(f)
-                sys.stderr.write('(none): E: error while reading %s: %s\n' % (f, e))
+                    arg = os.path.abspath(arg)
+                Pkg.warn('(none): E: error while reading %s: %s' % (arg, e))
                 pkgs = []
                 continue
 
@@ -141,37 +173,27 @@ def main():
                             elif do_spec_check and fname.endswith('.spec'):
                                 pkg = Pkg.FakePkg(fname)
                                 check = SpecCheck.SpecCheck()
+                                check.verbose = verbose
                                 check.check_spec(pkg, fname)
                                 pkg.cleanup()
                                 specfiles_checked += 1
 
                         except KeyboardInterrupt:
-                            sys.stderr.write('(none): E: interrupted, exiting while reading %s\n' % fname)
+                            Pkg.warn('(none): E: interrupted, exiting while ' +
+                                     'reading %s' % fname)
                             sys.exit(2)
                         except Exception, e:
-                            sys.stderr.write(
-                                '(none): E: while reading %s: %s\n' %
-                                (fname, e))
+                            Pkg.warn(
+                                '(none): E: while reading %s: %s' % (fname, e))
                             continue
             except Exception, e:
-                sys.stderr.write(
+                Pkg.warn(
                     '(none): E: error while reading dir %s: %s' % (dname, e))
                 continue
 
-        # if requested, scan all the installed packages
-        if allpkgs:
-            try:
-                ts = rpm.TransactionSet('/')
-                for hdr in ts.dbMatch():
-                    pkg = Pkg.InstalledPkg(hdr[rpm.RPMTAG_NAME], hdr)
-                    runChecks(pkg)
-                    packages_checked += 1
-            except KeyboardInterrupt:
-                sys.stderr.write('(none): E: interrupted, exiting while scanning all packages\n')
-                sys.exit(2)
-
         if printAllReasons():
-            sys.stderr.write('(none): E: badness %d exceeds threshold %d, aborting.\n' % (badnessScore(), badnessThreshold()))
+            Pkg.warn('(none): E: badness %d exceeds threshold %d, aborting.' %
+                     (badnessScore(), badnessThreshold()))
             sys.exit(66)
 
     finally:
@@ -192,10 +214,10 @@ def runChecks(pkg):
         for name in Config.allChecks():
             check = AbstractCheck.AbstractCheck.known_checks.get(name)
             if check:
+                check.verbose = verbose
                 check.check(pkg)
             else:
-                sys.stderr.write(
-                    '(none): W: unknown check %s, skipping\n' % name)
+                Pkg.warn('(none): W: unknown check %s, skipping' % name)
     finally:
         pkg.cleanup()
 
@@ -208,7 +230,7 @@ sys.argv[0] = os.path.basename(sys.argv[0])
 # parse options
 try:
     (opt, args) = getopt.getopt(sys.argv[1:],
-                              'iI:c:C:hVvanE:f:',
+                              'iI:c:C:hVvanE:f:o:',
                               ['info',
                                'check=',
                                'checkdir=',
@@ -219,20 +241,23 @@ try:
                                'noexception',
                                'extractdir=',
                                'file=',
+                               'option=',
                                ])
-except getopt.error, e:
-    sys.stderr.write("%s: %s\n" % (sys.argv[0], e))
+except getopt.GetoptError, e:
+    Pkg.warn("%s: %s" % (sys.argv[0], e))
     usage(sys.argv[0])
     sys.exit(1)
 
 # process options
 checkdir = '/usr/share/rpmlint'
 checks = []
-verbose = 0
+verbose = False
 extract_dir = None
-allpkgs = 0
-conf_file = '~/.rpmlintrc'
-info_error = 0
+conf_file = _default_user_conf
+if not os.path.exists(os.path.expanduser(conf_file)):
+    # deprecated backwards compatibility with < 0.88
+    conf_file = '~/.rpmlintrc'
+info_error = None
 
 # load global config files
 configs = glob.glob('/etc/rpmlint/*config')
@@ -244,39 +269,46 @@ for f in configs:
     except IOError:
         pass
     except Exception, E:
-        sys.stderr.write('(none): W: error loading %s, skipping: %s\n' % (f, E))
+        Pkg.warn('(none): W: error loading %s, skipping: %s' % (f, E))
 # pychecker fix
 del f
 
+config_overrides = {}
+
 # process command line options
 for o in opt:
-    if o[0] == '-c' or o[0] == '--check':
+    if o[0] in ('-c', '--check'):
         checks.append(o[1])
-    elif o[0] == '-i' or o[0] == '--info':
-        Config.info = 1
+    elif o[0] in('-i', '--info'):
+        Config.info = True
     elif o[0] == '-I':
         info_error = o[1]
-    elif o[0] == '-h' or o[0] == '--help':
+    elif o[0] in ('-h', '--help'):
         usage(sys.argv[0])
         sys.exit(0)
-    elif o[0] == '-C' or o[0] == '--checkdir':
+    elif o[0] in ('-C', '--checkdir'):
         Config.addCheckDir(o[1])
-    elif o[0] == '-v' or o[0] == '--verbose':
-        verbose = 1
-    elif o[0] == '-V' or o[0] == '--version':
+    elif o[0] in ('-v', '--verbose'):
+        verbose = True
+    elif o[0] in ('-V', '--version'):
         printVersion()
         sys.exit(0)
-    elif o[0] == '-E' or o[0] == '--extractdir':
+    elif o[0] in ('-E', '--extractdir'):
         extract_dir = o[1]
         Config.setOption('ExtractDir', extract_dir)
-    elif o[0] == '-n' or o[0] == '--noexception':
-        Config.no_exception = 1
-    elif o[0] == '-a' or o[0] == '--all':
-        allpkgs = 1
-    elif o[0] == '-f' or o[0] == '--file':
+    elif o[0] in ('-n', '--noexception'):
+        Config.no_exception = True
+    elif o[0] in ('-a', '--all'):
+        if '*' not in args:
+            args.append('*')
+    elif o[0] in ('-f', '--file'):
         conf_file = o[1]
-    else:
-        print 'unknown option', o
+    elif o[0] in ('-o', '--option'):
+        kv = o[1].split(None, 1)
+        if len(kv) == 1:
+            config_overrides[kv[0]] = None
+        else:
+            config_overrides[kv[0]] = eval(kv[1])
 
 # load user config file
 try:
@@ -284,13 +316,18 @@ try:
 except IOError:
     pass
 except Exception,E:
-    sys.stderr.write('(none): W: error loading %s, skipping: %s\n' % (conf_file, E))
+    Pkg.warn('(none): W: error loading %s, skipping: %s' % (conf_file, E))
+
+# apply config overrides
+for key, value in config_overrides.items():
+    Config.setOption(key, value)
 
 if not extract_dir:
     extract_dir = Config.getOption('ExtractDir', tempfile.gettempdir())
 
 if info_error:
-    Config.info = 1
+    Config.info = True
+    sys.path[0:0] = Config.checkDirs()
     for c in checks:
         Config.addCheck(c)
     for c in Config.allChecks():
@@ -301,7 +338,7 @@ if info_error:
     sys.exit(0)
 
 # if no argument print usage
-if args == [] and not allpkgs:
+if not args:
     usage(sys.argv[0])
     sys.exit(1)
 
